import cocotb
from cocotb.triggers import FallingEdge, Timer


async def generate_sysclk(dut):
    """Generate clock pulses."""

    while True:
        dut.i_sysclk.value = 1
        await Timer(5, units="ns")
        dut.i_sysclk.value = 0
        await Timer(5, units="ns")

async def generate_mii_tx_clk(dut):
    while True:
        dut.mii_tx_clk.value = 0
        await Timer(20, units="ns")
        dut.mii_tx_clk.value = 1
        await Timer(20, units="ns")

async def generate_mii_rx_clk(dut):
    while True:
        dut.mii_rx_clk.value = 0
        await Timer(20, units="ns")
        dut.mii_rx_clk.value = 1
        await Timer(20, units="ns")

async def handle_reset(dut):
    await cocotb.start(generate_sysclk(dut))  # run the clock "in the background"
    await cocotb.start(generate_mii_tx_clk(dut))  # run the clock "in the background"
    await cocotb.start(generate_mii_rx_clk(dut))  # run the clock "in the background"

    dut.io_asyncResetn.value = 1

    dut.mii_rx_dv.value = 0
    dut.mii_rx_er.value = 0
    dut.mii_rxd.value = 0


    for _ in range(10):
        await FallingEdge(dut.i_sysclk)  # wait for falling edge/"negedge"

    dut.io_asyncResetn.value = 0

    for _ in range(10):
        await FallingEdge(dut.i_sysclk)  # wait for falling edge/"negedge"

    dut.io_asyncResetn.value = 1

    await Timer(500, units="ns")  # wait a bit
    await FallingEdge(dut.i_sysclk)  # wait for falling edge/"negedge"



