from base_test import handle_reset
from phy_sim import traffic_generator, traffic_generator_from_hex

import struct

import cocotb
from cocotb.triggers import Timer, FallingEdge
from cocotb.handle import Force
from packet_gen import create_udp, create_arp



async def ext_loopback(dut):
    while True:
        await FallingEdge(dut.mii_tx_clk)  # wait for falling edge/"negedge"
        dut.mii_rxd.value = dut.mii_txd.value
        dut.mii_rx_dv.value = dut.mii_tx_en.value
        dut.mii_rx_er.value = dut.mii_tx_er.value


@cocotb.test()
async def send_packet_test(dut):
    await handle_reset(dut)

    for _ in range(4):
        await traffic_generator_from_hex(dut, "udp.hex")
        await Timer(1, units="us")

    for _ in range(4):
        await traffic_generator_from_hex(dut, "ping.hex")
        await Timer(1, units="us")

    await Timer(10, units="us")

@cocotb.test()
async def gen_packet_test(dut):
    await handle_reset(dut)

    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">LLL", 0, 0, 0), 41282, 8000)
    await traffic_generator(dut, data)
    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">bbbbLL", 1, 0, 0, 0, 0, 0xaa5500ff), 41282, 8000)
    await traffic_generator(dut, data)
    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">LLL", 0, 0, 0), 41282, 8000)
    await traffic_generator(dut, data)

    await Timer(20, units="us")

@cocotb.test()
async def arp_test(dut):
    await handle_reset(dut)

    data = create_arp()

    await traffic_generator(dut, data)
    await Timer(20, units="us")

@cocotb.test()
async def dhcp_test(dut):
    await handle_reset(dut)

    await traffic_generator_from_hex(dut, "dhcp.hex")
    await Timer(10, units="us")

    data = create_arp()
    await traffic_generator(dut, data)
    for _ in range(4):
        await traffic_generator_from_hex(dut, "ping.hex")

    await Timer(10, units="us")

@cocotb.test()
async def dhcp_followed_by_udp_test(dut):
    await handle_reset(dut)

    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">LLL", 0, 0, 0), 41282, 8000)
    await traffic_generator(dut, data)
    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">bbbbLL", 1, 0, 0, 0, 0, 0xaa5500ff), 41282, 8000)
    await traffic_generator(dut, data)
    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">LLL", 0, 0, 0), 41282, 8000)
    await traffic_generator(dut, data)

    await Timer(20, units="us")

    await traffic_generator_from_hex(dut, "dhcp.hex")
    await Timer(10, units="us")

    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">LLL", 0, 0, 0), 41282, 8000)
    await traffic_generator(dut, data)
    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">bbbbLL", 1, 0, 0, 0, 0, 0xaa5500ff), 41282, 8000)
    await traffic_generator(dut, data)
    data = create_udp("172.0.0.1", "172.0.0.2", "02:00:00:00:00:01", "02:00:00:aa:bb:cc", struct.pack(">LLL", 0, 0, 0), 41282, 8000)
    await traffic_generator(dut, data)

    await Timer(20, units="us")