from cocotb.triggers import FallingEdge

import zlib

async def traffic_generator_from_bin(dut, filename: str):
    with open(filename, "rb") as file:
        contents = file.read()
        await traffic_generator(dut, contents)

async def traffic_generator_from_hex(dut, filename: str):
    data = bytearray()

    with open(filename, "r") as file:
        for line in file:
            for word in line.split():
                if len(word) == 2: # make sure there are no gaps!
                    data.append(int(word, 16).to_bytes(1)[0])

    await traffic_generator(dut, data)

async def traffic_generator(dut, data: bytes):
    dut.mii_rx_dv.value = 1
    await preamble_sfd(dut)

    for byte in data:
        await write_byte(dut, byte)

    crc = zlib.crc32(data)
    for i in range(4):
        byte = (crc >> (8*i)) & 0xFF
        await write_byte(dut, byte)

    dut.mii_rx_dv.value = 0
    await write_byte(dut, 0)

async def preamble_sfd(dut):
    for _ in range(7):
        await write_byte(dut, 0x55)

    await write_byte(dut, 0xD5)

async def write_byte(dut, data: int):
    await FallingEdge(dut.mii_rx_clk)
    dut.mii_rxd.value = data & 0xf
    await FallingEdge(dut.mii_rx_clk)
    dut.mii_rxd.value = (data >> 4) & 0xf
