`timescale 1ns/1ps

module sim_top ();

logic clk_100;

// clk_100
initial begin
    clk_100 <= '1;
    forever begin
        #5 clk_100 <= ~clk_100;
    end
end

logic clk_25;

// clk_100
initial begin
    clk_25 <= '1;
    forever begin
        #20 clk_25 <= ~clk_25;
    end
end

initial begin
    $dumpfile("sim_top.vcd");
    $dumpvars(0,sim_top);
end

logic           reset;

logic   [31:0]  eth_ctrl_AXI4_AWADDR;
logic           eth_ctrl_AXI4_AWREADY;
logic           eth_ctrl_AXI4_AWVALID;
logic           eth_ctrl_AXI4_WREADY;
logic   [31:0]  eth_ctrl_AXI4_WDATA;
logic   [3:0]   eth_ctrl_AXI4_WSTRB;
logic           eth_ctrl_AXI4_WVALID;
logic           eth_ctrl_AXI4_BVALID;
logic           eth_ctrl_AXI4_BREADY;
logic   [1:0]   eth_ctrl_AXI4_BRESP;
logic           eth_ctrl_AXI4_ARVALID;
logic           eth_ctrl_AXI4_ARREADY;
logic   [31:0]  eth_ctrl_AXI4_ARADDR;
logic           eth_ctrl_AXI4_RREADY;
logic   [31:0]  eth_ctrl_AXI4_RDATA;
logic           eth_ctrl_AXI4_RVALID;
logic   [1:0]   eth_ctrl_AXI4_RRESP;

logic   [31:0]  eth_tx_dma_AXI4_AWADDR;
logic           eth_tx_dma_AXI4_AWREADY;
logic           eth_tx_dma_AXI4_AWVALID;
logic           eth_tx_dma_AXI4_WREADY;
logic   [31:0]  eth_tx_dma_AXI4_WDATA;
logic   [3:0]   eth_tx_dma_AXI4_WSTRB;
logic           eth_tx_dma_AXI4_WVALID;
logic           eth_tx_dma_AXI4_BVALID;
logic           eth_tx_dma_AXI4_BREADY;
logic   [1:0]   eth_tx_dma_AXI4_BRESP;
logic           eth_tx_dma_AXI4_ARVALID;
logic           eth_tx_dma_AXI4_ARREADY;
logic   [31:0]  eth_tx_dma_AXI4_ARADDR;
logic           eth_tx_dma_AXI4_RREADY;
logic   [31:0]  eth_tx_dma_AXI4_RDATA;
logic           eth_tx_dma_AXI4_RVALID;
logic   [1:0]   eth_tx_dma_AXI4_RRESP;

logic   [31:0]  eth_rx_dma_AXI4_AWADDR;
logic           eth_rx_dma_AXI4_AWREADY;
logic           eth_rx_dma_AXI4_AWVALID;
logic           eth_rx_dma_AXI4_WREADY;
logic   [31:0]  eth_rx_dma_AXI4_WDATA;
logic   [3:0]   eth_rx_dma_AXI4_WSTRB;
logic           eth_rx_dma_AXI4_WVALID;
logic           eth_rx_dma_AXI4_BVALID;
logic           eth_rx_dma_AXI4_BREADY;
logic   [1:0]   eth_rx_dma_AXI4_BRESP;
logic           eth_rx_dma_AXI4_ARVALID;
logic           eth_rx_dma_AXI4_ARREADY;
logic   [31:0]  eth_rx_dma_AXI4_ARADDR;
logic           eth_rx_dma_AXI4_RREADY;
logic   [31:0]  eth_rx_dma_AXI4_RDATA;
logic           eth_rx_dma_AXI4_RVALID;
logic   [1:0]   eth_rx_dma_AXI4_RRESP;

logic   [3:0]   mii_rxd;
logic   [3:0]   mii_txd;
logic           mii_rx_clk;
logic           mii_rx_dv;
logic           mii_rx_er;
logic           mii_tx_clk;
logic           mii_tx_en;
logic           mii_tx_er;

logic           ram_awvalid;
logic           ram_awready;
logic   [31:0]  ram_awaddr;
logic           ram_wvalid;
logic           ram_wready;
logic   [31:0]  ram_wdata;
logic   [3:0]   ram_wstrb;
logic           ram_bvalid;
logic           ram_bready;
logic   [1:0]   ram_bresp;
logic           ram_arvalid;
logic           ram_arready;
logic   [31:0]  ram_araddr;
logic           ram_rvalid;
logic           ram_rready;
logic   [31:0]  ram_rdata;
logic   [1:0]   ram_rresp;


task axi4lite_write(input logic [31:0] addr, input logic [31:0] data);
    eth_ctrl_AXI4_AWADDR = addr;
    eth_ctrl_AXI4_AWVALID = '1;
    @(posedge clk_100);
    while (!eth_ctrl_AXI4_AWREADY) begin
        @(posedge clk_100);
    end
    eth_ctrl_AXI4_AWADDR = '0;
    eth_ctrl_AXI4_AWVALID = '0;
    
    eth_ctrl_AXI4_BREADY = '1;

    eth_ctrl_AXI4_WDATA = data;
    eth_ctrl_AXI4_WVALID = '1;
    eth_ctrl_AXI4_WSTRB = '1;

    @(posedge clk_100);
    while (!eth_ctrl_AXI4_WREADY) begin
        @(posedge clk_100);
    end
    eth_ctrl_AXI4_WDATA = '0;
    eth_ctrl_AXI4_WVALID = '0;
    eth_ctrl_AXI4_WSTRB = '0;
endtask

task axi4lite_read(input logic [31:0] addr, output logic [31:0] data);
    eth_ctrl_AXI4_ARADDR = addr;
    eth_ctrl_AXI4_ARVALID = '1;

    @(posedge clk_100);
    while (!eth_ctrl_AXI4_ARREADY) begin
        @(posedge clk_100);
    end
    eth_ctrl_AXI4_ARADDR = '0;
    eth_ctrl_AXI4_ARVALID = '0;
    
    eth_ctrl_AXI4_RREADY = '1;

    @(posedge clk_100);
    while (!eth_ctrl_AXI4_RVALID) begin
        @(posedge clk_100);
    end
    data = eth_ctrl_AXI4_RDATA;
    eth_ctrl_AXI4_RREADY = '0;
endtask

eth_wrapper dut(
    .clk_100        (clk_100),
    .reset          (reset),

    .s_control_AXI4_AWADDR      (eth_ctrl_AXI4_AWADDR),
    .s_control_AXI4_AWREADY     (eth_ctrl_AXI4_AWREADY),
    .s_control_AXI4_AWVALID     (eth_ctrl_AXI4_AWVALID),
    .s_control_AXI4_WREADY      (eth_ctrl_AXI4_WREADY),
    .s_control_AXI4_WDATA       (eth_ctrl_AXI4_WDATA),
    .s_control_AXI4_WSTRB       (eth_ctrl_AXI4_WSTRB),
    .s_control_AXI4_WVALID      (eth_ctrl_AXI4_WVALID),
    .s_control_AXI4_BVALID      (eth_ctrl_AXI4_BVALID),
    .s_control_AXI4_BREADY      (eth_ctrl_AXI4_BREADY),
    .s_control_AXI4_BRESP       (eth_ctrl_AXI4_BRESP),
    .s_control_AXI4_ARVALID     (eth_ctrl_AXI4_ARVALID),
    .s_control_AXI4_ARREADY     (eth_ctrl_AXI4_ARREADY),
    .s_control_AXI4_ARADDR      (eth_ctrl_AXI4_ARADDR),
    .s_control_AXI4_RREADY      (eth_ctrl_AXI4_RREADY),
    .s_control_AXI4_RDATA       (eth_ctrl_AXI4_RDATA),
    .s_control_AXI4_RVALID      (eth_ctrl_AXI4_RVALID),
    .s_control_AXI4_RRESP       (eth_ctrl_AXI4_RRESP),

    .m_tx_dma_AXI4_AWADDR       (eth_tx_dma_AXI4_AWADDR),
    .m_tx_dma_AXI4_AWREADY      (eth_tx_dma_AXI4_AWREADY),
    .m_tx_dma_AXI4_AWVALID      (eth_tx_dma_AXI4_AWVALID),
    .m_tx_dma_AXI4_WREADY       (eth_tx_dma_AXI4_WREADY),
    .m_tx_dma_AXI4_WDATA        (eth_tx_dma_AXI4_WDATA),
    .m_tx_dma_AXI4_WSTRB        (eth_tx_dma_AXI4_WSTRB),
    .m_tx_dma_AXI4_WVALID       (eth_tx_dma_AXI4_WVALID),
    .m_tx_dma_AXI4_BVALID       (eth_tx_dma_AXI4_BVALID),
    .m_tx_dma_AXI4_BREADY       (eth_tx_dma_AXI4_BREADY),
    .m_tx_dma_AXI4_BRESP        (eth_tx_dma_AXI4_BRESP),
    .m_tx_dma_AXI4_ARVALID      (eth_tx_dma_AXI4_ARVALID),
    .m_tx_dma_AXI4_ARREADY      (eth_tx_dma_AXI4_ARREADY),
    .m_tx_dma_AXI4_ARADDR       (eth_tx_dma_AXI4_ARADDR),
    .m_tx_dma_AXI4_RREADY       (eth_tx_dma_AXI4_RREADY),
    .m_tx_dma_AXI4_RDATA        (eth_tx_dma_AXI4_RDATA),
    .m_tx_dma_AXI4_RVALID       (eth_tx_dma_AXI4_RVALID),
    .m_tx_dma_AXI4_RRESP        (eth_tx_dma_AXI4_RRESP),

    .m_rx_dma_AXI4_AWADDR       (eth_rx_dma_AXI4_AWADDR),
    .m_rx_dma_AXI4_AWREADY      (eth_rx_dma_AXI4_AWREADY),
    .m_rx_dma_AXI4_AWVALID      (eth_rx_dma_AXI4_AWVALID),
    .m_rx_dma_AXI4_WREADY       (eth_rx_dma_AXI4_WREADY),
    .m_rx_dma_AXI4_WDATA        (eth_rx_dma_AXI4_WDATA),
    .m_rx_dma_AXI4_WSTRB        (eth_rx_dma_AXI4_WSTRB),
    .m_rx_dma_AXI4_WVALID       (eth_rx_dma_AXI4_WVALID),
    .m_rx_dma_AXI4_BVALID       (eth_rx_dma_AXI4_BVALID),
    .m_rx_dma_AXI4_BREADY       (eth_rx_dma_AXI4_BREADY),
    .m_rx_dma_AXI4_BRESP        (eth_rx_dma_AXI4_BRESP),
    .m_rx_dma_AXI4_ARVALID      (eth_rx_dma_AXI4_ARVALID),
    .m_rx_dma_AXI4_ARREADY      (eth_rx_dma_AXI4_ARREADY),
    .m_rx_dma_AXI4_ARADDR       (eth_rx_dma_AXI4_ARADDR),
    .m_rx_dma_AXI4_RREADY       (eth_rx_dma_AXI4_RREADY),
    .m_rx_dma_AXI4_RDATA        (eth_rx_dma_AXI4_RDATA),
    .m_rx_dma_AXI4_RVALID       (eth_rx_dma_AXI4_RVALID),
    .m_rx_dma_AXI4_RRESP        (eth_rx_dma_AXI4_RRESP),

    .mii_rx_clk                 (mii_rx_clk),
    .mii_rxd                    (mii_rxd),
    .mii_rx_dv                  (mii_rx_dv),
    .mii_rx_er                  (mii_rx_er),
    .mii_tx_clk                 (mii_tx_clk),
    .mii_txd                    (mii_txd),
    .mii_tx_en                  (mii_tx_en),
    .mii_tx_er                  (mii_tx_er)
);

axi4_lite_ram #(
    .RAM_SIZE(8),
    .ZERO_INIT(0),
    .RAM_INIT_FILE("init_hex.mem")
) u_ram(
    .i_clk(clk_100),
    .i_rst(reset),

    .o_AWREADY(ram_awready),
    .o_WREADY(ram_wready),

    .o_BVALID(ram_bvalid),
    .i_BREADY(ram_bready),
    .o_BRESP(ram_bresp),

    .i_ARVALID(ram_arvalid),
    .o_ARREADY(ram_arready),
    .i_ARADDR(ram_araddr),
    .i_ARPROT('0),

    .o_RVALID(ram_rvalid),
    .i_RREADY(ram_rready),
    .o_RDATA(ram_rdata),
    .o_RRESP(ram_rresp),

    .i_AWVALID(ram_awvalid),
    .i_AWADDR(ram_awaddr),
    .i_AWPROT('0),

    .i_WVALID(ram_wvalid),
    .i_WDATA(ram_wdata),
    .i_WSTRB(ram_wstrb)
);


axilxbar #(
    .NM(2),
    .NS(1),
    .SLAVE_ADDR(
        {32'hff, 32'h0}
    )
) u_axilxbar (
    .S_AXI_ACLK     (clk_100),
    .S_AXI_ARESETN  (~reset),

    .S_AXI_AWVALID  ({eth_tx_dma_AXI4_AWVALID,    eth_rx_dma_AXI4_AWVALID }),
    .S_AXI_AWREADY  ({eth_tx_dma_AXI4_AWREADY,    eth_rx_dma_AXI4_AWREADY }),
    .S_AXI_AWADDR   ({eth_tx_dma_AXI4_AWADDR,     eth_rx_dma_AXI4_AWADDR  }),
    .S_AXI_AWPROT   ('0),
    .S_AXI_WVALID   ({eth_tx_dma_AXI4_WVALID,     eth_rx_dma_AXI4_WVALID  }),
    .S_AXI_WREADY   ({eth_tx_dma_AXI4_WREADY,     eth_rx_dma_AXI4_WREADY  }),
    .S_AXI_WDATA    ({eth_tx_dma_AXI4_WDATA,      eth_rx_dma_AXI4_WDATA   }),
    .S_AXI_WSTRB    ({eth_tx_dma_AXI4_WSTRB,      eth_rx_dma_AXI4_WSTRB   }),
    .S_AXI_BVALID   ({eth_tx_dma_AXI4_BVALID,     eth_rx_dma_AXI4_BVALID  }),
    .S_AXI_BREADY   ({eth_tx_dma_AXI4_BREADY,     eth_rx_dma_AXI4_BREADY  }),
    .S_AXI_BRESP    ({eth_tx_dma_AXI4_BRESP,      eth_rx_dma_AXI4_BRESP   }),
    .S_AXI_ARVALID  ({eth_tx_dma_AXI4_ARVALID,    eth_rx_dma_AXI4_ARVALID }),
    .S_AXI_ARREADY  ({eth_tx_dma_AXI4_ARREADY,    eth_rx_dma_AXI4_ARREADY }),
    .S_AXI_ARADDR   ({eth_tx_dma_AXI4_ARADDR,     eth_rx_dma_AXI4_ARADDR  }),
    .S_AXI_ARPROT   ('0),
    .S_AXI_RVALID   ({eth_tx_dma_AXI4_RVALID,     eth_rx_dma_AXI4_RVALID  }),
    .S_AXI_RREADY   ({eth_tx_dma_AXI4_RREADY,     eth_rx_dma_AXI4_RREADY  }),
    .S_AXI_RDATA    ({eth_tx_dma_AXI4_RDATA,      eth_rx_dma_AXI4_RDATA   }),
    .S_AXI_RRESP    ({eth_tx_dma_AXI4_RRESP,      eth_rx_dma_AXI4_RRESP   }),

    .M_AXI_AWADDR   (ram_awaddr),
    .M_AXI_AWPROT   (),
    .M_AXI_AWVALID  (ram_awvalid),
    .M_AXI_AWREADY  (ram_awready),
    .M_AXI_WDATA    (ram_wdata),
    .M_AXI_WSTRB    (ram_wstrb),
    .M_AXI_WVALID   (ram_wvalid),
    .M_AXI_WREADY   (ram_wready),
    .M_AXI_BRESP    (ram_bresp),
    .M_AXI_BVALID   (ram_bvalid),
    .M_AXI_BREADY   (ram_bready),
    .M_AXI_ARADDR   (ram_araddr),
    .M_AXI_ARPROT   (),
    .M_AXI_ARVALID  (ram_arvalid),
    .M_AXI_ARREADY  (ram_arready),
    .M_AXI_RDATA    (ram_rdata),
    .M_AXI_RRESP    (ram_rresp),
    .M_AXI_RVALID   (ram_rvalid),
    .M_AXI_RREADY   (ram_rready)
);

assign mii_rx_clk = clk_25;
assign mii_tx_clk = clk_25;

assign mii_rxd = '0;
assign mii_rx_dv = '0;

assign mii_rx_er = '0;

initial begin
    reset = '1;
    repeat (5) @(posedge clk_100);
    reset = '0;
    repeat (5) @(posedge clk_100);

    axi4lite_write(32'h4000044, 32'h1);

    axi4lite_write(32'h400002c, 32'h1);

    axi4lite_write(32'h4000000, 32'h00000000);
    axi4lite_write(32'h4000004, 32'h80);
    axi4lite_write(32'h400000c, 32'h1);


    repeat (5000) @(posedge clk_100);
    $finish();
end

endmodule
