`timescale 1ns/1ps

module sim_top ();

logic clk_100;

// clk_100
initial begin
    clk_100 <= '1;
    forever begin
        #5 clk_100 <= ~clk_100;
    end
end

logic clk_25;

// clk_100
initial begin
    clk_25 <= '1;
    forever begin
        #20 clk_25 <= ~clk_25;
    end
end

initial begin
    $dumpfile("sim_top.vcd");
    $dumpvars(0,sim_top);
end

task apb_write(input logic [31:0] addr, input logic [31:0] data);
    m_apb_psel = '1;
    m_apb_pwrite = '1;
    m_apb_paddr = addr;
    m_apb_pwdata = data;
    m_apb_pstrb = '1;
    @(posedge clk_100);
    m_apb_penable = '1;
    @(posedge clk_100);
    m_apb_penable = '0;
    m_apb_psel = '0;
endtask

task apb_read(input logic [31:0] addr, output logic [31:0] data);
    m_apb_psel = '1;
    m_apb_pwrite = '0;
    m_apb_paddr = addr;
    m_apb_pstrb = '1;
    @(posedge clk_100);
    m_apb_penable = '1;
    @(posedge clk_100);
    data = m_apb_prdata;
    m_apb_penable = '0;
    m_apb_psel = '0;
endtask

logic reset;

logic           m_apb_psel;
logic           m_apb_penable;
logic           m_apb_pwrite;
logic   [2:0]   m_apb_pprot;
logic   [31:0]  m_apb_paddr;
logic   [31:0]  m_apb_pwdata;
logic   [3:0]   m_apb_pstrb;
logic           m_apb_pready;
logic   [31:0]  m_apb_prdata;
logic           m_apb_pslverr;

logic           s_apb_s2m_psel;
logic           s_apb_s2m_penable;
logic           s_apb_s2m_pwrite;
logic   [2:0]   s_apb_s2m_pprot;
logic   [31:0]  s_apb_s2m_paddr;
logic   [31:0]  s_apb_s2m_pwdata;
logic   [3:0]   s_apb_s2m_pstrb;
logic           s_apb_s2m_pready;
logic   [31:0]  s_apb_s2m_prdata;
logic           s_apb_s2m_pslverr;

logic           s_apb_m2s_psel;
logic           s_apb_m2s_penable;
logic           s_apb_m2s_pwrite;
logic   [2:0]   s_apb_m2s_pprot;
logic   [31:0]  s_apb_m2s_paddr;
logic   [31:0]  s_apb_m2s_pwdata;
logic   [3:0]   s_apb_m2s_pstrb;
logic           s_apb_m2s_pready;
logic   [31:0]  s_apb_m2s_prdata;
logic           s_apb_m2s_pslverr;


logic           ram_awvalid;
logic           ram_awready;
logic   [31:0]  ram_awaddr;
logic           ram_wvalid;
logic           ram_wready;
logic   [31:0]  ram_wdata;
logic   [3:0]   ram_wstrb;
logic           ram_bvalid;
logic           ram_bready;
logic   [1:0]   ram_bresp;
logic           ram_arvalid;
logic           ram_arready;
logic   [31:0]  ram_araddr;
logic           ram_rvalid;
logic           ram_rready;
logic   [31:0]  ram_rdata;
logic   [1:0]   ram_rresp;


logic   [31:0]  eth_tx_dma_AXI4_AWADDR;
logic           eth_tx_dma_AXI4_AWREADY;
logic           eth_tx_dma_AXI4_AWVALID;
logic           eth_tx_dma_AXI4_WREADY;
logic   [31:0]  eth_tx_dma_AXI4_WDATA;
logic   [3:0]   eth_tx_dma_AXI4_WSTRB;
logic           eth_tx_dma_AXI4_WVALID;
logic           eth_tx_dma_AXI4_BVALID;
logic           eth_tx_dma_AXI4_BREADY;
logic   [1:0]   eth_tx_dma_AXI4_BRESP;
logic           eth_tx_dma_AXI4_ARVALID;
logic           eth_tx_dma_AXI4_ARREADY;
logic   [31:0]  eth_tx_dma_AXI4_ARADDR;
logic           eth_tx_dma_AXI4_RREADY;
logic   [31:0]  eth_tx_dma_AXI4_RDATA;
logic           eth_tx_dma_AXI4_RVALID;
logic   [1:0]   eth_tx_dma_AXI4_RRESP;

logic   [31:0]  eth_rx_dma_AXI4_AWADDR;
logic           eth_rx_dma_AXI4_AWREADY;
logic           eth_rx_dma_AXI4_AWVALID;
logic           eth_rx_dma_AXI4_WREADY;
logic   [31:0]  eth_rx_dma_AXI4_WDATA;
logic   [3:0]   eth_rx_dma_AXI4_WSTRB;
logic           eth_rx_dma_AXI4_WVALID;
logic           eth_rx_dma_AXI4_BVALID;
logic           eth_rx_dma_AXI4_BREADY;
logic   [1:0]   eth_rx_dma_AXI4_BRESP;
logic           eth_rx_dma_AXI4_ARVALID;
logic           eth_rx_dma_AXI4_ARREADY;
logic   [31:0]  eth_rx_dma_AXI4_ARADDR;
logic           eth_rx_dma_AXI4_RREADY;
logic   [31:0]  eth_rx_dma_AXI4_RDATA;
logic           eth_rx_dma_AXI4_RVALID;
logic   [1:0]   eth_rx_dma_AXI4_RRESP;

logic   [7:0]   tx_axis_tdata;
logic           tx_axis_tvalid;
logic           tx_axis_tready;
logic           tx_axis_tlast;
logic           tx_axis_tuser;
logic           tx_axis_tkeep;

logic   [7:0]   rx_axis_tdata;
logic           rx_axis_tvalid;
logic           rx_axis_tready;
logic           rx_axis_tlast;
logic           rx_axis_tuser;
logic           rx_axis_tkeep;

logic   [3:0]   mii_rxd;
logic   [3:0]   mii_txd;
logic           mii_rx_clk;
logic           mii_rx_dv;
logic           mii_rx_er;
logic           mii_tx_clk;
logic           mii_tx_en;
logic           mii_tx_er;

assign mii_rxd = mii_txd;

assign mii_rx_dv = mii_tx_en;


m2s_dma tx_dma (
    .clk                    (clk_100),
    .rst                    (reset),

    .s_apb_psel             (s_apb_m2s_psel),
    .s_apb_penable          (s_apb_m2s_penable),
    .s_apb_pwrite           (s_apb_m2s_pwrite),
    .s_apb_pprot            (s_apb_m2s_pprot),
    .s_apb_paddr            (s_apb_m2s_paddr),
    .s_apb_pwdata           (s_apb_m2s_pwdata),
    .s_apb_pstrb            (s_apb_m2s_pstrb),
    .s_apb_pready           (s_apb_m2s_pready),
    .s_apb_prdata           (s_apb_m2s_prdata),
    .s_apb_pslverr          (s_apb_m2s_pslverr),

    .m_dma_AXI4_AWADDR      (eth_tx_dma_AXI4_AWADDR),
    .m_dma_AXI4_AWREADY     (eth_tx_dma_AXI4_AWREADY),
    .m_dma_AXI4_AWVALID     (eth_tx_dma_AXI4_AWVALID),
    .m_dma_AXI4_WREADY      (eth_tx_dma_AXI4_WREADY),
    .m_dma_AXI4_WDATA       (eth_tx_dma_AXI4_WDATA),
    .m_dma_AXI4_WSTRB       (eth_tx_dma_AXI4_WSTRB),
    .m_dma_AXI4_WVALID      (eth_tx_dma_AXI4_WVALID),
    .m_dma_AXI4_BVALID      (eth_tx_dma_AXI4_BVALID),
    .m_dma_AXI4_BREADY      (eth_tx_dma_AXI4_BREADY),
    .m_dma_AXI4_BRESP       (eth_tx_dma_AXI4_BRESP),
    .m_dma_AXI4_ARVALID     (eth_tx_dma_AXI4_ARVALID),
    .m_dma_AXI4_ARREADY     (eth_tx_dma_AXI4_ARREADY),
    .m_dma_AXI4_ARADDR      (eth_tx_dma_AXI4_ARADDR),
    .m_dma_AXI4_RREADY      (eth_tx_dma_AXI4_RREADY),
    .m_dma_AXI4_RDATA       (eth_tx_dma_AXI4_RDATA),
    .m_dma_AXI4_RVALID      (eth_tx_dma_AXI4_RVALID),
    .m_dma_AXI4_RRESP       (eth_tx_dma_AXI4_RRESP),

    .tx_axis_tdata          (tx_axis_tdata),
    .tx_axis_tvalid         (tx_axis_tvalid),
    .tx_axis_tready         (tx_axis_tready),
    .tx_axis_tlast          (tx_axis_tlast),
    .tx_axis_tuser          (tx_axis_tuser)
);

s2m_dma rx_dma (
    .clk                    (clk_100),
    .rst                    (reset),

    .s_apb_psel             (s_apb_s2m_psel),
    .s_apb_penable          (s_apb_s2m_penable),
    .s_apb_pwrite           (s_apb_s2m_pwrite),
    .s_apb_pprot            (s_apb_s2m_pprot),
    .s_apb_paddr            (s_apb_s2m_paddr),
    .s_apb_pwdata           (s_apb_s2m_pwdata),
    .s_apb_pstrb            (s_apb_s2m_pstrb),
    .s_apb_pready           (s_apb_s2m_pready),
    .s_apb_prdata           (s_apb_s2m_prdata),
    .s_apb_pslverr          (s_apb_s2m_pslverr),

    .m_dma_AXI4_AWADDR      (eth_rx_dma_AXI4_AWADDR),
    .m_dma_AXI4_AWREADY     (eth_rx_dma_AXI4_AWREADY),
    .m_dma_AXI4_AWVALID     (eth_rx_dma_AXI4_AWVALID),
    .m_dma_AXI4_WREADY      (eth_rx_dma_AXI4_WREADY),
    .m_dma_AXI4_WDATA       (eth_rx_dma_AXI4_WDATA),
    .m_dma_AXI4_WSTRB       (eth_rx_dma_AXI4_WSTRB),
    .m_dma_AXI4_WVALID      (eth_rx_dma_AXI4_WVALID),
    .m_dma_AXI4_BVALID      (eth_rx_dma_AXI4_BVALID),
    .m_dma_AXI4_BREADY      (eth_rx_dma_AXI4_BREADY),
    .m_dma_AXI4_BRESP       (eth_rx_dma_AXI4_BRESP),
    .m_dma_AXI4_ARVALID     (eth_rx_dma_AXI4_ARVALID),
    .m_dma_AXI4_ARREADY     (eth_rx_dma_AXI4_ARREADY),
    .m_dma_AXI4_ARADDR      (eth_rx_dma_AXI4_ARADDR),
    .m_dma_AXI4_RREADY      (eth_rx_dma_AXI4_RREADY),
    .m_dma_AXI4_RDATA       (eth_rx_dma_AXI4_RDATA),
    .m_dma_AXI4_RVALID      (eth_rx_dma_AXI4_RVALID),
    .m_dma_AXI4_RRESP       (eth_rx_dma_AXI4_RRESP),

    .rx_axis_tdata          (rx_axis_tdata),
    .rx_axis_tvalid         (rx_axis_tvalid),
    .rx_axis_tready         (rx_axis_tready),
    .rx_axis_tlast          (rx_axis_tlast),
    .rx_axis_tuser          (rx_axis_tuser)
);

eth_mac_mii_fifo #(
    .ENABLE_PADDING(1),
    .MIN_FRAME_LENGTH(64),
    .AXIS_DATA_WIDTH(8)
) u_eth_mac(
    .rst                (reset),
    .logic_clk          (clk_100),
    .logic_rst          (reset),

    .tx_axis_tdata      (tx_axis_tdata),
    .tx_axis_tkeep      ('1),
    .tx_axis_tvalid     (tx_axis_tvalid),
    .tx_axis_tready     (tx_axis_tready),
    .tx_axis_tlast      (tx_axis_tlast),
    .tx_axis_tuser      ('0),

    .rx_axis_tdata      (rx_axis_tdata),
    .rx_axis_tkeep      (),
    .rx_axis_tvalid     (rx_axis_tvalid),
    .rx_axis_tready     (rx_axis_tready),
    .rx_axis_tlast      (rx_axis_tlast),
    .rx_axis_tuser      (rx_axis_tuser),

    .mii_rx_clk         (clk_25),
    .mii_rxd            (mii_rxd),
    .mii_rx_dv          (mii_rx_dv),
    .mii_rx_er          (mii_rx_er),
    .mii_tx_clk         (clk_25),
    .mii_txd            (mii_txd),
    .mii_tx_en          (mii_tx_en),
    .mii_tx_er          (mii_tx_er),

    .tx_error_underflow (),
    .tx_fifo_overflow   (),
    .tx_fifo_bad_frame  (),
    .tx_fifo_good_frame (),
    .rx_error_bad_frame (),
    .rx_error_bad_fcs   (),
    .rx_fifo_overflow   (),
    .rx_fifo_bad_frame  (),
    .rx_fifo_good_frame (),

    .cfg_ifg            ('0),
    .cfg_tx_enable      ('1),   // this should be configurable w/ regfile
    .cfg_rx_enable      ('1)    
);

axi4_lite_ram #(
    .RAM_SIZE(8),
    .ZERO_INIT(0),
    .RAM_INIT_FILE("init_hex.mem")
) u_ram(
    .i_clk(clk_100),
    .i_rst(reset),

    .o_AWREADY(ram_awready),
    .o_WREADY(ram_wready),

    .o_BVALID(ram_bvalid),
    .i_BREADY(ram_bready),
    .o_BRESP(ram_bresp),

    .i_ARVALID(ram_arvalid),
    .o_ARREADY(ram_arready),
    .i_ARADDR(ram_araddr),
    .i_ARPROT('0),

    .o_RVALID(ram_rvalid),
    .i_RREADY(ram_rready),
    .o_RDATA(ram_rdata),
    .o_RRESP(ram_rresp),

    .i_AWVALID(ram_awvalid),
    .i_AWADDR(ram_awaddr),
    .i_AWPROT('0),

    .i_WVALID(ram_wvalid),
    .i_WDATA(ram_wdata),
    .i_WSTRB(ram_wstrb)
);


axilxbar #(
    .NM(2),
    .NS(1),
    .SLAVE_ADDR(
        {32'hff, 32'h0}
    )
) u_axilxbar (
    .S_AXI_ACLK     (clk_100),
    .S_AXI_ARESETN  (~reset),

    .S_AXI_AWVALID  ({eth_tx_dma_AXI4_AWVALID,    eth_rx_dma_AXI4_AWVALID }),
    .S_AXI_AWREADY  ({eth_tx_dma_AXI4_AWREADY,    eth_rx_dma_AXI4_AWREADY }),
    .S_AXI_AWADDR   ({eth_tx_dma_AXI4_AWADDR,     eth_rx_dma_AXI4_AWADDR  }),
    .S_AXI_AWPROT   ('0),
    .S_AXI_WVALID   ({eth_tx_dma_AXI4_WVALID,     eth_rx_dma_AXI4_WVALID  }),
    .S_AXI_WREADY   ({eth_tx_dma_AXI4_WREADY,     eth_rx_dma_AXI4_WREADY  }),
    .S_AXI_WDATA    ({eth_tx_dma_AXI4_WDATA,      eth_rx_dma_AXI4_WDATA   }),
    .S_AXI_WSTRB    ({eth_tx_dma_AXI4_WSTRB,      eth_rx_dma_AXI4_WSTRB   }),
    .S_AXI_BVALID   ({eth_tx_dma_AXI4_BVALID,     eth_rx_dma_AXI4_BVALID  }),
    .S_AXI_BREADY   ({eth_tx_dma_AXI4_BREADY,     eth_rx_dma_AXI4_BREADY  }),
    .S_AXI_BRESP    ({eth_tx_dma_AXI4_BRESP,      eth_rx_dma_AXI4_BRESP   }),
    .S_AXI_ARVALID  ({eth_tx_dma_AXI4_ARVALID,    eth_rx_dma_AXI4_ARVALID }),
    .S_AXI_ARREADY  ({eth_tx_dma_AXI4_ARREADY,    eth_rx_dma_AXI4_ARREADY }),
    .S_AXI_ARADDR   ({eth_tx_dma_AXI4_ARADDR,     eth_rx_dma_AXI4_ARADDR  }),
    .S_AXI_ARPROT   ('0),
    .S_AXI_RVALID   ({eth_tx_dma_AXI4_RVALID,     eth_rx_dma_AXI4_RVALID  }),
    .S_AXI_RREADY   ({eth_tx_dma_AXI4_RREADY,     eth_rx_dma_AXI4_RREADY  }),
    .S_AXI_RDATA    ({eth_tx_dma_AXI4_RDATA,      eth_rx_dma_AXI4_RDATA   }),
    .S_AXI_RRESP    ({eth_tx_dma_AXI4_RRESP,      eth_rx_dma_AXI4_RRESP   }),

    .M_AXI_AWADDR   (ram_awaddr),
    .M_AXI_AWPROT   (),
    .M_AXI_AWVALID  (ram_awvalid),
    .M_AXI_AWREADY  (ram_awready),
    .M_AXI_WDATA    (ram_wdata),
    .M_AXI_WSTRB    (ram_wstrb),
    .M_AXI_WVALID   (ram_wvalid),
    .M_AXI_WREADY   (ram_wready),
    .M_AXI_BRESP    (ram_bresp),
    .M_AXI_BVALID   (ram_bvalid),
    .M_AXI_BREADY   (ram_bready),
    .M_AXI_ARADDR   (ram_araddr),
    .M_AXI_ARPROT   (),
    .M_AXI_ARVALID  (ram_arvalid),
    .M_AXI_ARREADY  (ram_arready),
    .M_AXI_RDATA    (ram_rdata),
    .M_AXI_RRESP    (ram_rresp),
    .M_AXI_RVALID   (ram_rvalid),
    .M_AXI_RREADY   (ram_rready)
);

apb4_decoder #(
    .NS(2),
    .SLAVE_ADDR({
        {32'h1f, 32'h0},
        {32'h3f, 32'h20}
    })
) u_apb4_decoder (
    .s_apb_psel     (m_apb_psel),
    .s_apb_penable  (m_apb_penable),
    .s_apb_pwrite   (m_apb_pwrite),
    .s_apb_pprot    (m_apb_pprot),
    .s_apb_paddr    (m_apb_paddr),
    .s_apb_pwdata   (m_apb_pwdata),
    .s_apb_pstrb    (m_apb_pstrb),
    .s_apb_pready   (m_apb_pready),
    .s_apb_prdata   (m_apb_prdata),
    .s_apb_pslverr  (m_apb_pslverr),

    .m_apb_psel     ({s_apb_m2s_psel,       s_apb_s2m_psel      }),
    .m_apb_penable  ({s_apb_m2s_penable,    s_apb_s2m_penable   }),
    .m_apb_pwrite   ({s_apb_m2s_pwrite,     s_apb_s2m_pwrite    }),
    .m_apb_pprot    ({s_apb_m2s_pprot,      s_apb_s2m_pprot     }),
    .m_apb_paddr    ({s_apb_m2s_paddr,      s_apb_s2m_paddr     }),
    .m_apb_pwdata   ({s_apb_m2s_pwdata,     s_apb_s2m_pwdata    }),
    .m_apb_pstrb    ({s_apb_m2s_pstrb,      s_apb_s2m_pstrb     }),
    .m_apb_pready   ({s_apb_m2s_pready,     s_apb_s2m_pready    }),
    .m_apb_prdata   ({s_apb_m2s_prdata,     s_apb_s2m_prdata    }),
    .m_apb_pslverr  ({s_apb_m2s_pslverr,    s_apb_s2m_pslverr   })
);

initial begin
    reset = '1;

    m_apb_psel = '0;
    m_apb_penable = '0;
    m_apb_pwrite = '0;
    m_apb_pprot = '0;
    m_apb_paddr = '0;
    m_apb_pwdata = '0;
    m_apb_pstrb = '0;

    mii_rx_er = '0;

    repeat (5) @(posedge clk_100);
    reset = '0;
    repeat (5) @(posedge clk_100);

    apb_write(32'h2c, 32'h1);

    apb_write(0, 32'h00000000);
    apb_write(32'h4, 32'h80);
    apb_write(32'hc, 32'h1);

    repeat (5000) @(posedge clk_100);

    $finish();

end

endmodule