`timescale 1ns/1ps

module sim_top ();

logic clk_100;

// clk_100
initial begin
    clk_100 <= '1;
    forever begin
        #5 clk_100 <= ~clk_100;
    end
end

logic clk_25;

// clk_100
initial begin
    clk_25 <= '1;
    forever begin
        #20 clk_25 <= ~clk_25;
    end
end

initial begin
    $dumpfile("sim_top.vcd");
    $dumpvars(0,sim_top);
end

task apb_write(input logic [5:0] addr, input logic [31:0] data);
    s_apb_psel = '1;
    s_apb_pwrite = '1;
    s_apb_paddr = addr;
    s_apb_pwdata = data;
    s_apb_pstrb = '1;
    @(posedge clk_100);
    s_apb_penable = '1;
    @(posedge clk_100);

    s_apb_penable = '0;
    s_apb_psel = '0;
endtask

task apb_read(input logic [5:0] addr, output logic [31:0] data);
    s_apb_psel = '1;
    s_apb_pwrite = '0;
    s_apb_paddr = addr;
    s_apb_pstrb = '1;
    @(posedge clk_100);
    s_apb_penable = '1;
    @(posedge clk_100);
    data = s_apb_prdata;
    s_apb_penable = '0;
    s_apb_psel = '0;
endtask



logic reset;

logic s_apb_psel;
logic s_apb_penable;
logic s_apb_pwrite;
logic [2:0] s_apb_pprot;
logic [5:0] s_apb_paddr;
logic [31:0] s_apb_pwdata;
logic [3:0] s_apb_pstrb;
logic s_apb_pready;
logic [31:0] s_apb_prdata;
logic s_apb_pslverr;


logic [7:0] tdata;
logic tvalid, tready, tlast, tuser, tkeep;

logic [7:0] rx_axis_mac_tdata;
logic rx_axis_mac_tkeep;
logic rx_axis_mac_tvalid;
logic rx_axis_mac_tready;
logic rx_axis_mac_tlast;
logic rx_axis_mac_tuser;

m2s_dma dut(
    .clk                    (clk_100),
    .rst                    (reset),

    .s_apb_psel             (s_apb_psel),
    .s_apb_penable          (s_apb_penable),
    .s_apb_pwrite           (s_apb_pwrite),
    .s_apb_pprot            (s_apb_pprot),
    .s_apb_paddr            (s_apb_paddr),
    .s_apb_pwdata           (s_apb_pwdata),
    .s_apb_pstrb            (s_apb_pstrb),
    .s_apb_pready           (s_apb_pready),
    .s_apb_prdata           (s_apb_prdata),
    .s_apb_pslverr          (s_apb_pslverr),

    .m_dma_AXI4_AWADDR      (ram_awaddr),
    .m_dma_AXI4_AWREADY     (ram_awready),
    .m_dma_AXI4_AWVALID     (ram_awvalid),
    .m_dma_AXI4_WREADY      (ram_wready),
    .m_dma_AXI4_WDATA       (ram_wdata),
    .m_dma_AXI4_WSTRB       (ram_wstrb),
    .m_dma_AXI4_WVALID      (ram_wvalid),
    .m_dma_AXI4_BVALID      (ram_bvalid),
    .m_dma_AXI4_BREADY      (ram_bready),
    .m_dma_AXI4_BRESP       (ram_bresp),
    .m_dma_AXI4_ARVALID     (ram_arvalid),
    .m_dma_AXI4_ARREADY     (ram_arready),
    .m_dma_AXI4_ARADDR      (ram_araddr),
    .m_dma_AXI4_RREADY      (ram_rready),
    .m_dma_AXI4_RDATA       (ram_rdata),
    .m_dma_AXI4_RVALID      (ram_rvalid),
    .m_dma_AXI4_RRESP       (ram_rresp),

    .tx_axis_tdata          (tdata),
    .tx_axis_tvalid         (tvalid),
    .tx_axis_tready         (tready),
    .tx_axis_tlast          (tlast),
    .tx_axis_tuser          (tuser)
);

logic                       ram_awvalid;
logic                       ram_awready;
logic [31:0]                ram_awaddr;
logic                       ram_wvalid;
logic                       ram_wready;
logic [31:0]                ram_wdata;
logic [3:0]                 ram_wstrb;
logic                       ram_bvalid;
logic                       ram_bready;
logic [1:0]                 ram_bresp;
logic                       ram_arvalid;
logic                       ram_arready;
logic [31:0]                ram_araddr;
logic                       ram_rvalid;
logic                       ram_rready;
logic [31:0]                ram_rdata;
logic [1:0]                 ram_rresp;


axi4_lite_ram #(
    .RAM_SIZE(8),
    .ZERO_INIT(0),
    .RAM_INIT_FILE("init_hex.mem")
) u_ram(
    .i_clk(clk_100),
    .i_rst(reset),

    .o_AWREADY(ram_awready),
    .o_WREADY(ram_wready),

    .o_BVALID(ram_bvalid),
    .i_BREADY(ram_bready),
    .o_BRESP(ram_bresp),

    .i_ARVALID(ram_arvalid),
    .o_ARREADY(ram_arready),
    .i_ARADDR(ram_araddr),
    .i_ARPROT('0),

    .o_RVALID(ram_rvalid),
    .i_RREADY(ram_rready),
    .o_RDATA(ram_rdata),
    .o_RRESP(ram_rresp),

    .i_AWVALID(ram_awvalid),
    .i_AWADDR(ram_awaddr),
    .i_AWPROT('0),

    .i_WVALID(ram_wvalid),
    .i_WDATA(ram_wdata),
    .i_WSTRB(ram_wstrb)
);

assign mii_rxd = mii_txd;

assign mii_rx_dv = mii_tx_en;


logic [3:0] mii_rxd;
logic [3:0] mii_txd;
logic mii_rx_clk;
logic mii_rx_dv;
logic mii_rx_er;
logic mii_tx_clk;
logic mii_tx_en;
logic mii_tx_er;

localparam AXIS_DATA_WIDTH = 8;

eth_mac_mii_fifo #(
    .ENABLE_PADDING(1),
    .MIN_FRAME_LENGTH(64),
    .AXIS_DATA_WIDTH(AXIS_DATA_WIDTH)
) u_eth_mac(
    .rst                (reset),
    .logic_clk          (clk_100),
    .logic_rst          (reset),

    .tx_axis_tdata      (tdata),
    .tx_axis_tkeep      (tvalid),
    .tx_axis_tvalid     (tvalid),
    .tx_axis_tready     (tready),
    .tx_axis_tlast      (tlast),
    .tx_axis_tuser      ('0),

    .rx_axis_tdata      (rx_axis_mac_tdata),
    .rx_axis_tkeep      (rx_axis_mac_tkeep),
    .rx_axis_tvalid     (rx_axis_mac_tvalid),
    .rx_axis_tready     (rx_axis_mac_tready),
    .rx_axis_tlast      (rx_axis_mac_tlast),
    .rx_axis_tuser      (rx_axis_mac_tuser),

    .mii_rx_clk         (clk_25),
    .mii_rxd            (mii_rxd),
    .mii_rx_dv          (mii_rx_dv),
    .mii_rx_er          (mii_rx_er),
    .mii_tx_clk         (clk_25),
    .mii_txd            (mii_txd),
    .mii_tx_en          (mii_tx_en),
    .mii_tx_er          (mii_tx_er),

    .tx_error_underflow (),
    .tx_fifo_overflow   (),
    .tx_fifo_bad_frame  (),
    .tx_fifo_good_frame (),
    .rx_error_bad_frame (),
    .rx_error_bad_fcs   (),
    .rx_fifo_overflow   (),
    .rx_fifo_bad_frame  (),
    .rx_fifo_good_frame (),

    .cfg_ifg            ('0),
    .cfg_tx_enable      ('1),   // this should be configurable w/ regfile
    .cfg_rx_enable      ('1)    
);


initial begin
    reset = '1;
    s_apb_psel = '0;
    s_apb_penable = '0;
    s_apb_pwrite = '0;
    s_apb_pprot = '0;
    s_apb_paddr = '0;
    s_apb_pwdata = '0;
    s_apb_pstrb = '0;

    mii_rx_er = '0;

    rx_axis_mac_tready = '1;

    repeat (5) @(posedge clk_100);
    reset = '0;
    repeat (5) @(posedge clk_100);

    apb_write(0, 32'h00000000);
    apb_write(4'h4, 32'h80);
    apb_write(4'hc, 32'h1);

    repeat (2000) @(posedge clk_100);
    $finish();
end


endmodule