`timescale 1ns/1ps

module sim_top();

logic clk_100;

// clk_100
initial begin
    clk_100 <= '1;
    forever begin
        #5 clk_100 <= ~clk_100;
    end
end

logic clk_25;

// clk_100
initial begin
    clk_25 <= '1;
    forever begin
        #20 clk_25 <= ~clk_25;
    end
end


logic [3:0] mii_rxd;
logic [3:0] mii_txd;
logic mii_rx_clk;
logic mii_rx_dv;
logic mii_rx_er;
logic mii_tx_clk;
logic mii_tx_en;
logic mii_tx_er;

logic reset;

logic pat_gen_en;
logic [15:0] pat_gen_num;
logic [15:0] pat_gen_ipg;

logic [47:0] dst_mac;
logic [47:0] src_mac;
logic [15:0] mac_dlen;

logic [7:0] tdata;
logic tkeep;
logic tvalid;
logic tlast;
logic tready;

logic [7:0] rx_axis_mac_tdata;
logic rx_axis_mac_tkeep;
logic rx_axis_mac_tvalid;
logic rx_axis_mac_tready;
logic rx_axis_mac_tlast;
logic rx_axis_mac_tuser;

assign mii_rxd = mii_txd;

assign mii_rx_dv = mii_tx_en;

localparam AXIS_DATA_WIDTH = 8;

eth_mac_mii_fifo #(
    .ENABLE_PADDING(1),
    .MIN_FRAME_LENGTH(64),
    .AXIS_DATA_WIDTH(AXIS_DATA_WIDTH)
) u_eth_mac(
    .rst                (reset),
    .logic_clk          (clk_100),
    .logic_rst          (reset),

    .tx_axis_tdata      (tdata),
    .tx_axis_tkeep      (tkeep),
    .tx_axis_tvalid     (tvalid),
    .tx_axis_tready     (tready),
    .tx_axis_tlast      (tlast),
    .tx_axis_tuser      ('0),

    .rx_axis_tdata      (rx_axis_mac_tdata),
    .rx_axis_tkeep      (rx_axis_mac_tkeep),
    .rx_axis_tvalid     (rx_axis_mac_tvalid),
    .rx_axis_tready     (rx_axis_mac_tready),
    .rx_axis_tlast      (rx_axis_mac_tlast),
    .rx_axis_tuser      (rx_axis_mac_tuser),

    .mii_rx_clk         (clk_25),
    .mii_rxd            (mii_rxd),
    .mii_rx_dv          (mii_rx_dv),
    .mii_rx_er          (mii_rx_er),
    .mii_tx_clk         (clk_25),
    .mii_txd            (mii_txd),
    .mii_tx_en          (mii_tx_en),
    .mii_tx_er          (mii_tx_er),

    .tx_error_underflow (),
    .tx_fifo_overflow   (),
    .tx_fifo_bad_frame  (),
    .tx_fifo_good_frame (),
    .rx_error_bad_frame (),
    .rx_error_bad_fcs   (),
    .rx_fifo_overflow   (),
    .rx_fifo_bad_frame  (),
    .rx_fifo_good_frame (),

    .cfg_ifg            ('0),
    .cfg_tx_enable      ('1),   // this should be configurable w/ regfile
    .cfg_rx_enable      ('1)    
);

mac_pat_gen u_mac_pat_gen(
    .clk(clk_100),
    .rstn(~reset),
    .pat_gen_en(pat_gen_en),
    .pat_gen_num(pat_gen_num),
    .pat_gen_ipg(pat_gen_ipg),
    .dst_mac(dst_mac),
    .src_mac(src_mac),
    .mac_dlen(mac_dlen),
    .rclk(),
    .rrstn(),
    .rdata(),
    .rvalid(),
    .rlast(),
    .tdata(tdata),
    .tvalid(tvalid),
    .tlast(tlast),
    .tready(tready)
);

initial begin
    $dumpfile("sim_top.vcd");
    $dumpvars(0,sim_top);
end

initial begin
    reset = '1;
    pat_gen_en = '0;
    pat_gen_num = 0;
    pat_gen_ipg = 16'h400;
    tkeep = '0;

    rx_axis_mac_tready = '1;

    dst_mac = 48'haabbccddeeff;
    src_mac = 48'h112233445566;
    mac_dlen = 16'h40;

    mii_rx_er = '0;


    repeat (5) @(posedge clk_100);
    reset = '0;
    repeat (5) @(posedge clk_100);
    pat_gen_en = '1;
    repeat (1000) @(posedge clk_100);
    $finish();
end

endmodule