`timescale 1ns/1ps

module sim_top ();

logic clk_100;

// clk_100
initial begin
    clk_100 <= '1;
    forever begin
        #5 clk_100 <= ~clk_100;
    end
end

initial begin
    $dumpfile("sim_top.vcd");
    $dumpvars(0,sim_top);
end

task apb_write(input logic [5:0] addr, input logic [31:0] data);
    s_apb_psel = '1;
    s_apb_pwrite = '1;
    s_apb_paddr = addr;
    s_apb_pwdata = data;
    s_apb_pstrb = '1;
    @(posedge clk_100);
    s_apb_penable = '1;
    @(posedge clk_100);

    s_apb_penable = '0;
    s_apb_psel = '0;
endtask

task apb_read(input logic [5:0] addr, output logic [31:0] data);
    s_apb_psel = '1;
    s_apb_pwrite = '0;
    s_apb_paddr = addr;
    s_apb_pstrb = '1;
    @(posedge clk_100);
    s_apb_penable = '1;
    @(posedge clk_100);
    data = s_apb_prdata;
    s_apb_penable = '0;
    s_apb_psel = '0;
endtask

logic reset;

logic s_apb_psel;
logic s_apb_penable;
logic s_apb_pwrite;
logic [2:0] s_apb_pprot;
logic [5:0] s_apb_paddr;
logic [31:0] s_apb_pwdata;
logic [3:0] s_apb_pstrb;
logic s_apb_pready;
logic [31:0] s_apb_prdata;
logic s_apb_pslverr;

logic pat_gen_en;
logic [15:0] pat_gen_num;
logic [15:0] pat_gen_ipg;

logic [47:0] dst_mac;
logic [47:0] src_mac;
logic [15:0] mac_dlen;

logic                       ram_awvalid;
logic                       ram_awready;
logic [31:0]                ram_awaddr;
logic                       ram_wvalid;
logic                       ram_wready;
logic [31:0]                ram_wdata;
logic [3:0]                 ram_wstrb;
logic                       ram_bvalid;
logic                       ram_bready;
logic [1:0]                 ram_bresp;
logic                       ram_arvalid;
logic                       ram_arready;
logic [31:0]                ram_araddr;
logic                       ram_rvalid;
logic                       ram_rready;
logic [31:0]                ram_rdata;
logic [1:0]                 ram_rresp;

logic [7:0] tdata;
logic tvalid, tready, tlast, tuser, tkeep;

s2m_dma dut(
    .clk                    (clk_100),
    .rst                    (reset),

    .s_apb_psel             (s_apb_psel),
    .s_apb_penable          (s_apb_penable),
    .s_apb_pwrite           (s_apb_pwrite),
    .s_apb_pprot            (s_apb_pprot),
    .s_apb_paddr            (s_apb_paddr),
    .s_apb_pwdata           (s_apb_pwdata),
    .s_apb_pstrb            (s_apb_pstrb),
    .s_apb_pready           (s_apb_pready),
    .s_apb_prdata           (s_apb_prdata),
    .s_apb_pslverr          (s_apb_pslverr),

    .m_dma_AXI4_AWADDR      (ram_awaddr),
    .m_dma_AXI4_AWREADY     (ram_awready),
    .m_dma_AXI4_AWVALID     (ram_awvalid),
    .m_dma_AXI4_WREADY      (ram_wready),
    .m_dma_AXI4_WDATA       (ram_wdata),
    .m_dma_AXI4_WSTRB       (ram_wstrb),
    .m_dma_AXI4_WVALID      (ram_wvalid),
    .m_dma_AXI4_BVALID      (ram_bvalid),
    .m_dma_AXI4_BREADY      (ram_bready),
    .m_dma_AXI4_BRESP       (ram_bresp),
    .m_dma_AXI4_ARVALID     (ram_arvalid),
    .m_dma_AXI4_ARREADY     (ram_arready),
    .m_dma_AXI4_ARADDR      (ram_araddr),
    .m_dma_AXI4_RREADY      (ram_rready),
    .m_dma_AXI4_RDATA       (ram_rdata),
    .m_dma_AXI4_RVALID      (ram_rvalid),
    .m_dma_AXI4_RRESP       (ram_rresp),

    .rx_axis_tdata          (tdata),
    .rx_axis_tvalid         (tvalid),
    .rx_axis_tready         (tready),
    .rx_axis_tlast          (tlast),
    .rx_axis_tuser          (tuser)
);

mac_pat_gen u_mac_pat_gen(
    .clk(clk_100),
    .rstn(~reset),
    .pat_gen_en(pat_gen_en),
    .pat_gen_num(pat_gen_num),
    .pat_gen_ipg(pat_gen_ipg),
    .dst_mac(dst_mac),
    .src_mac(src_mac),
    .mac_dlen(mac_dlen),
    .rclk(),
    .rrstn(),
    .rdata(),
    .rvalid(),
    .rlast(),
    .tdata(tdata),
    .tvalid(tvalid),
    .tlast(tlast),
    .tready(tready)
);


axi4_lite_ram #(
    .RAM_SIZE(8),
    .ZERO_INIT(1)
) u_ram(
    .i_clk(clk_100),
    .i_rst(reset),

    .o_AWREADY(ram_awready),
    .o_WREADY(ram_wready),

    .o_BVALID(ram_bvalid),
    .i_BREADY(ram_bready),
    .o_BRESP(ram_bresp),

    .i_ARVALID(ram_arvalid),
    .o_ARREADY(ram_arready),
    .i_ARADDR(ram_araddr),
    .i_ARPROT('0),

    .o_RVALID(ram_rvalid),
    .i_RREADY(ram_rready),
    .o_RDATA(ram_rdata),
    .o_RRESP(ram_rresp),

    .i_AWVALID(ram_awvalid),
    .i_AWADDR(ram_awaddr),
    .i_AWPROT('0),

    .i_WVALID(ram_wvalid),
    .i_WDATA(ram_wdata),
    .i_WSTRB(ram_wstrb)
);

initial begin
    reset = '1;

    pat_gen_en = '0;
    pat_gen_num = 0;
    pat_gen_ipg = 16'h40;

    dst_mac = 48'haabbccddeeff;
    src_mac = 48'h112233445566;
    mac_dlen = 16'h40;

    tuser = '0;

    repeat (5) @(posedge clk_100);
    reset = '0;
    repeat (5) @(posedge clk_100);
    pat_gen_en = '1;
    repeat (5) @(posedge clk_100);

    apb_write(4'hc, 32'h1);

    repeat (500) @(posedge clk_100);

    $finish();
end

endmodule