`timescale 1ns/1ps

module sim_top();

`include "include/sdram_controller_define.vh"

localparam ADDR_WIDTH = 32;
localparam DATA_WIDTH = 32;

logic clk_100, clk_200;

// clk_100
initial begin
    clk_100 <= '1;
    forever begin
        #5 clk_100 <= ~clk_100;
    end
end

// clk_200
initial begin
    clk_200 <= '1;
    forever begin
        #2.5 clk_200 <= ~clk_200;
    end
end


initial begin
    $dumpfile("sim_top.vcd");
    $dumpvars(0,sim_top);
end

logic button_reset;

wire    [DQ_GROUP	*DQ_WIDTH	-1:0]   w_sdr_DQ;
// ^ Has to be wire because of tristate/inout stuff

logic   [31:0]  sdram_AXI4_AWADDR;
logic           sdram_AXI4_AWREADY;
logic           sdram_AXI4_AWVALID;
logic           sdram_AXI4_WREADY;
logic   [31:0]  sdram_AXI4_WDATA;
logic   [3:0]   sdram_AXI4_WSTRB;
logic           sdram_AXI4_WVALID;
logic           sdram_AXI4_BVALID;
logic           sdram_AXI4_BREADY;
logic           sdram_AXI4_ARVALID;
logic           sdram_AXI4_ARREADY;
logic   [31:0]  sdram_AXI4_ARADDR;
logic           sdram_AXI4_RREADY;
logic   [31:0]  sdram_AXI4_RDATA;
logic           sdram_AXI4_RVALID;

genvar i, j;
generate
    for (i=0; i<DQ_GROUP*DQ_WIDTH; i=i+1)
    begin: DQ_map
        assign	w_sdr_DQ[i]	=	(w_sdr_DATA_oe[i]) ? w_sdr_DATA[i] : 1'bz;
    end

    for (j=0; j<DQ_GROUP; j=j+1)
    begin : mem_inst
        generic_sdr inst_sdr
        (
            .Dq(w_sdr_DQ[((j+1)*(DQ_WIDTH))-1:((j)*DQ_WIDTH)]),
            .Addr(o_sdr_ADDR[ROW_WIDTH-1:0]),
            .Ba(o_sdr_BA[BA_WIDTH-1:0]),
            .Clk(~clk_200),
            .Cke(o_sdr_CKE),
            .Cs_n(o_sdr_n_CS),
            .Ras_n(o_sdr_n_RAS),
            .Cas_n(o_sdr_n_CAS),
            .We_n(o_sdr_n_WE),
            .Dqm(o_sdr_DQM[j])
        );
    end
endgenerate

logic           o_sdr_CKE;
logic           o_sdr_n_CS;
logic           o_sdr_n_WE;
logic           o_sdr_n_RAS;
logic           o_sdr_n_CAS;
logic [1:0]     o_sdr_BA;
logic [12:0]    o_sdr_ADDR;
logic [15:0]    i_sdr_DATA;
logic [15:0]    o_sdr_DATA;
logic [15:0]    o_sdr_DATA_oe;
logic [1:0]     o_sdr_DQM;

logic [1:0] w_sdr_CKE;
logic [1:0] w_sdr_n_CS;
logic [1:0] w_sdr_n_RAS;
logic [1:0] w_sdr_n_CAS;
logic [1:0] w_sdr_n_WE;
logic [3:0] w_sdr_BA;
logic [25:0] w_sdr_ADDR;
logic [31:0] w_sdr_DATA;
logic [31:0] w_sdr_DATA_oe;
logic [3:0] w_sdr_DQM;

assign i_sdr_data = {16'h0, w_sdr_DQ};
assign o_sdr_CKE = w_sdr_CKE[0];    //Using SOFT ddio, ignore second cycle
assign o_sdr_n_CS = w_sdr_n_CS[0];
assign o_sdr_n_RAS = w_sdr_n_RAS[0];
assign o_sdr_n_CAS = w_sdr_n_CAS[0];
assign o_sdr_n_WE = w_sdr_n_WE[0];
assign o_sdr_BA = w_sdr_BA[0+:2];
assign o_sdr_ADDR = w_sdr_ADDR[0+:13];
assign o_sdr_DATA = w_sdr_DATA[0+:16];
assign o_sdr_DATA_oe = w_sdr_DATA_oe[0+:16];
assign o_sdr_DQM = w_sdr_DQM[0+:2];

sdram_controller u_sdram_controller(
    .i_aresetn                          (button_reset),

    .i_sysclk                           (clk_100),
    .i_sdrclk                           (clk_200),
    .i_tACclk                           (clk_200),

    .o_pll_reset                        (),
    .i_pll_locked                       ('1),
    
    
    .i_AXI4_AWADDR                      ((sdram_AXI4_AWADDR - 32'h1000000) >> 0),
    .o_AXI4_AWREADY                     (sdram_AXI4_AWREADY),
    .i_AXI4_AWVALID                     (sdram_AXI4_AWVALID),
    .o_AXI4_WREADY                      (sdram_AXI4_WREADY),
    .i_AXI4_WDATA                       (sdram_AXI4_WDATA),
    .i_AXI4_WSTRB                       (sdram_AXI4_WSTRB),
    .i_AXI4_WLAST                       (sdram_AXI4_WVALID),
    .i_AXI4_WVALID                      (sdram_AXI4_WVALID),
    .o_AXI4_BVALID                      (sdram_AXI4_BVALID),
    .i_AXI4_BREADY                      (sdram_AXI4_BREADY),
    .i_AXI4_ARVALID                     (sdram_AXI4_ARVALID),
    .o_AXI4_ARREADY                     (sdram_AXI4_ARREADY),
    .i_AXI4_ARADDR                      ((sdram_AXI4_ARADDR - 32'h1000000) >> 0),
    .i_AXI4_RREADY                      (sdram_AXI4_RREADY),
    .o_AXI4_RDATA                       (sdram_AXI4_RDATA),
    .o_AXI4_RLAST                       (),
    .o_AXI4_RVALID                      (sdram_AXI4_RVALID),
    .i_AXI4_AWID                        ('0),
    .i_AXI4_AWSIZE                      ('0),
    .i_AXI4_ARID                        ('0),
    .i_AXI4_ARLEN                       ('0),
    .i_AXI4_ARSIZE                      ('0),
    .i_AXI4_ARBURST                     ('0),
    .i_AXI4_AWLEN                       ('0),
    .o_AXI4_RID                         (),
    .o_AXI4_BID                         (),

    .o_sdr_init_done                    (),

    .o_ref_req                          (),

    .o_sdr_CKE                          (w_sdr_CKE),
    .o_sdr_n_CS                         (w_sdr_n_CS),
    .o_sdr_n_RAS                        (w_sdr_n_RAS),
    .o_sdr_n_CAS                        (w_sdr_n_CAS),
    .o_sdr_n_WE                         (w_sdr_n_WE),
    .o_sdr_BA                           (w_sdr_BA),
    .o_sdr_ADDR                         (w_sdr_ADDR),
    .o_sdr_DATA                         (w_sdr_DATA),
    .o_sdr_DATA_oe                      (w_sdr_DATA_oe),
    .i_sdr_DATA                         ({{16'b0}, {i_sdr_DATA}}),
    .o_sdr_DQM                          (w_sdr_DQM)
);




initial begin
    button_reset <= '1;
    repeat(10) @(clk_100);
    button_reset <= '0;

    sdram_AXI4_ARADDR = '0;
    sdram_AXI4_ARVALID = '0;
    sdram_AXI4_AWADDR = '0;
    sdram_AXI4_AWVALID = '0;
    sdram_AXI4_WDATA = '0;
    sdram_AXI4_RREADY = '1;
    sdram_AXI4_BREADY = '1;

    repeat(10) @(clk_100);
    button_reset <= '1;
    repeat(21000) @(posedge clk_100);


    sdram_AXI4_AWADDR = 32'h1fffffc;
    sdram_AXI4_AWVALID = '1;
    sdram_AXI4_WDATA = 32'haa5500ff;
    sdram_AXI4_WVALID = '1;

    @(posedge clk_100);
    while (~sdram_AXI4_WREADY) begin
        @(posedge clk_100);
    end

    sdram_AXI4_AWADDR = '0;
    sdram_AXI4_AWVALID = '0;
    sdram_AXI4_WDATA = '0;

    repeat (10) @(posedge clk_100);


    sdram_AXI4_AWADDR = 32'h2000000;
    sdram_AXI4_AWVALID = '1;
    sdram_AXI4_WDATA = 32'h12345678;
    sdram_AXI4_WVALID = '1;

    $display("Second AXI");
    @(posedge clk_100);
    $display("Second AXI after clock");
    while (~sdram_AXI4_WREADY) begin
        @(posedge clk_100);
    end

    sdram_AXI4_AWADDR = '0;
    sdram_AXI4_AWVALID = '0;
    sdram_AXI4_WDATA = '0;


    repeat(100) @(posedge clk_100);
    $finish();
end

endmodule