module audio_qam_fpga(
    input       io_asyncResetn,
    input logic i_sysclk,

    //MII Interface
    input   wire                    mii_rx_clk,
    input   wire    [3:0]           mii_rxd,
    input   wire                    mii_rx_dv,
    input   wire                    mii_rx_er,
    input   wire                    mii_tx_clk,
    output  wire    [3:0]           mii_txd,
    output  wire                    mii_tx_en,
    output  wire                    mii_tx_er,

    // MDIO Interface
    input                           i_Mdi,
    output                          o_Mdo,
    output                          o_MdoEn,
    output                          o_Mdc,

    output                          phy_rstn
);

    localparam AXIS_DATA_WIDTH = 8;
    localparam AXIS_KEEP_WIDTH = AXIS_DATA_WIDTH/8;


    assign phy_rstn = io_asyncResetn;


    wire reset;
    assign reset = ~io_asyncResetn;

    logic   [AXIS_DATA_WIDTH-1:0]   eth_tx_axis_tdata;
    logic                           eth_tx_axis_tvalid;
    logic                           eth_tx_axis_tready;
    logic                           eth_tx_axis_tlast;
    logic                           eth_tx_axis_tuser;
    logic   [AXIS_KEEP_WIDTH-1:0]   eth_tx_axis_tkeep;

    logic   [AXIS_DATA_WIDTH-1:0]   eth_rx_axis_tdata;
    logic                           eth_rx_axis_tvalid;
    logic                           eth_rx_axis_tready;
    logic                           eth_rx_axis_tlast;
    logic                           eth_rx_axis_tuser;
    logic   [AXIS_KEEP_WIDTH-1:0]   eth_rx_axis_tkeep;

    assign eth_tx_axis_tuser = '0;

    logic [31:0]                  m_apb_paddr;
    logic                         m_apb_psel;
    logic                         m_apb_penabl;
    logic                         m_apb_pwrite;
    logic [31:0]                  m_apb_pwdata;
    logic [3:0]                   m_apb_pstrb;
    logic                         m_apb_pready;
    logic [31:0]                  m_apb_prdata;
    logic                         m_apb_pslverr;

    eth_wrapper #(.DATA_WIDTH(AXIS_DATA_WIDTH)) u_eth_wrapper (
        .clk_100                            (i_sysclk),
        .reset                              (reset),    // TODO: Replace this with rstm

        // MII
        .mii_rx_clk                         (mii_rx_clk),
        .mii_rxd                            (mii_rxd),
        .mii_rx_dv                          (mii_rx_dv),
        .mii_rx_er                          (mii_rx_er),
        .mii_tx_clk                         (mii_tx_clk),
        .mii_txd                            (mii_txd),
        .mii_tx_en                          (mii_tx_en),
        .mii_tx_er                          (mii_tx_er),

        .tx_axis_tdata                      (eth_tx_axis_tdata),
        .tx_axis_tvalid                     (eth_tx_axis_tvalid),
        .tx_axis_tready                     (eth_tx_axis_tready),
        .tx_axis_tlast                      (eth_tx_axis_tlast),
        .tx_axis_tuser                      (eth_tx_axis_tuser),
        .tx_axis_tkeep                      (eth_tx_axis_tkeep),

        .rx_axis_tdata                      (eth_rx_axis_tdata),
        .rx_axis_tvalid                     (eth_rx_axis_tvalid),
        .rx_axis_tready                     (eth_rx_axis_tready),
        .rx_axis_tlast                      (eth_rx_axis_tlast),
        .rx_axis_tuser                      (eth_rx_axis_tuser),
        .rx_axis_tkeep                      (eth_rx_axis_tkeep),

        .Mdi                                (i_Mdi),
        .Mdo                                (o_Mdo),
        .MdoEn                              (o_MdoEn),
        .Mdc                                (o_Mdc)
    );

    network_top #(.AXIS_DATA_WIDTH(AXIS_DATA_WIDTH)) u_network_top(
        .clk                                (i_sysclk),
        .rst                                (reset),

        .eth_rx_axis_tdata                  (eth_rx_axis_tdata),
        .eth_rx_axis_tvalid                 (eth_rx_axis_tvalid),
        .eth_rx_axis_tready                 (eth_rx_axis_tready),
        .eth_rx_axis_tlast                  (eth_rx_axis_tlast),
        .eth_rx_axis_tkeep                  (eth_rx_axis_tkeep),

        .eth_tx_axis_tdata                  (eth_tx_axis_tdata),
        .eth_tx_axis_tvalid                 (eth_tx_axis_tvalid),
        .eth_tx_axis_tready                 (eth_tx_axis_tready),
        .eth_tx_axis_tlast                  (eth_tx_axis_tlast),
        .eth_tx_axis_tkeep                  (eth_tx_axis_tkeep),

        .m_apb_paddr                        (m_apb_paddr),
        .m_apb_psel                         (m_apb_psel),
        .m_apb_penable                      (m_apb_penable),
        .m_apb_pwrite                       (m_apb_pwrite),
        .m_apb_pwdata                       (m_apb_pwdata),
        .m_apb_pstrb                        (m_apb_pstrb),
        .m_apb_pready                       (m_apb_pready),
        .m_apb_prdata                       (m_apb_prdata),
        .m_apb_pslverr                      (m_apb_pslverr)
    );

    reg_top u_reg_top (
        .clk                                (i_sysclk),
        .rst                                (reset),

        .s_apb_psel                         (m_apb_psel),
        .s_apb_penable                      (m_apb_penable),
        .s_apb_pwrite                       (m_apb_pwrite),
        .s_apb_paddr                        (m_apb_paddr),
        .s_apb_pwdata                       (m_apb_pwdata),
        .s_apb_pready                       (m_apb_pready),
        .s_apb_prdata                       (m_apb_prdata),
        .s_apb_pslverr                      (m_apb_pslverr)
    );

endmodule


