// Generated by PeakRDL-regblock - A free and open-source SystemVerilog generator
//  https://github.com/SystemRDL/PeakRDL-regblock

module mac_regs (
        input wire clk,
        input wire rst,

        input wire s_apb_psel,
        input wire s_apb_penable,
        input wire s_apb_pwrite,
        input wire [2:0] s_apb_pprot,
        input wire [2:0] s_apb_paddr,
        input wire [31:0] s_apb_pwdata,
        input wire [3:0] s_apb_pstrb,
        output logic s_apb_pready,
        output logic [31:0] s_apb_prdata,
        output logic s_apb_pslverr,

        input mac_regs_pkg::mac_regs__in_t hwif_in,
        output mac_regs_pkg::mac_regs__out_t hwif_out
    );

    //--------------------------------------------------------------------------
    // CPU Bus interface logic
    //--------------------------------------------------------------------------
    logic cpuif_req;
    logic cpuif_req_is_wr;
    logic [2:0] cpuif_addr;
    logic [31:0] cpuif_wr_data;
    logic [31:0] cpuif_wr_biten;
    logic cpuif_req_stall_wr;
    logic cpuif_req_stall_rd;

    logic cpuif_rd_ack;
    logic cpuif_rd_err;
    logic [31:0] cpuif_rd_data;

    logic cpuif_wr_ack;
    logic cpuif_wr_err;

    // Request
    logic is_active;
    always_ff @(posedge clk) begin
        if(rst) begin
            is_active <= '0;
            cpuif_req <= '0;
            cpuif_req_is_wr <= '0;
            cpuif_addr <= '0;
            cpuif_wr_data <= '0;
            cpuif_wr_biten <= '0;
        end else begin
            if(~is_active) begin
                if(s_apb_psel) begin
                    is_active <= '1;
                    cpuif_req <= '1;
                    cpuif_req_is_wr <= s_apb_pwrite;
                    cpuif_addr <= {s_apb_paddr[2:2], 2'b0};
                    cpuif_wr_data <= s_apb_pwdata;
                    for(int i=0; i<4; i++) begin
                        cpuif_wr_biten[i*8 +: 8] <= {8{s_apb_pstrb[i]}};
                    end
                end
            end else begin
                cpuif_req <= '0;
                if(cpuif_rd_ack || cpuif_wr_ack) begin
                    is_active <= '0;
                end
            end
        end
    end

    // Response
    assign s_apb_pready = cpuif_rd_ack | cpuif_wr_ack;
    assign s_apb_prdata = cpuif_rd_data;
    assign s_apb_pslverr = cpuif_rd_err | cpuif_wr_err;

    logic cpuif_req_masked;

    // Read & write latencies are balanced. Stalls not required
    assign cpuif_req_stall_rd = '0;
    assign cpuif_req_stall_wr = '0;
    assign cpuif_req_masked = cpuif_req
                            & !(!cpuif_req_is_wr & cpuif_req_stall_rd)
                            & !(cpuif_req_is_wr & cpuif_req_stall_wr);

    //--------------------------------------------------------------------------
    // Address Decode
    //--------------------------------------------------------------------------
    typedef struct packed {
        logic mac_status;
        logic mii_loopback;
    } decoded_reg_strb_t;
    decoded_reg_strb_t decoded_reg_strb;
    logic decoded_req;
    logic decoded_req_is_wr;
    logic [31:0] decoded_wr_data;
    logic [31:0] decoded_wr_biten;

    always_comb begin
        decoded_reg_strb.mac_status = cpuif_req_masked & (cpuif_addr == 3'h0);
        decoded_reg_strb.mii_loopback = cpuif_req_masked & (cpuif_addr == 3'h4);
    end

    // Pass down signals to next stage
    assign decoded_req = cpuif_req_masked;
    assign decoded_req_is_wr = cpuif_req_is_wr;
    assign decoded_wr_data = cpuif_wr_data;
    assign decoded_wr_biten = cpuif_wr_biten;

    //--------------------------------------------------------------------------
    // Field logic
    //--------------------------------------------------------------------------
    typedef struct packed {
        struct packed {
            struct packed {
                logic next;
                logic load_next;
            } tx_error_underflow;
            struct packed {
                logic next;
                logic load_next;
            } tx_fifo_overflow;
            struct packed {
                logic next;
                logic load_next;
            } tx_fifo_bad_frame;
            struct packed {
                logic next;
                logic load_next;
            } tx_fifo_good_frame;
            struct packed {
                logic next;
                logic load_next;
            } rx_error_bad_frame;
            struct packed {
                logic next;
                logic load_next;
            } rx_error_bad_fcs;
            struct packed {
                logic next;
                logic load_next;
            } rx_fifo_overflow;
            struct packed {
                logic next;
                logic load_next;
            } rx_fifo_bad_frame;
            struct packed {
                logic next;
                logic load_next;
            } rx_fifo_good_frame;
        } mac_status;
        struct packed {
            struct packed {
                logic next;
                logic load_next;
            } mii_loopback_en;
        } mii_loopback;
    } field_combo_t;
    field_combo_t field_combo;

    typedef struct packed {
        struct packed {
            struct packed {
                logic value;
            } tx_error_underflow;
            struct packed {
                logic value;
            } tx_fifo_overflow;
            struct packed {
                logic value;
            } tx_fifo_bad_frame;
            struct packed {
                logic value;
            } tx_fifo_good_frame;
            struct packed {
                logic value;
            } rx_error_bad_frame;
            struct packed {
                logic value;
            } rx_error_bad_fcs;
            struct packed {
                logic value;
            } rx_fifo_overflow;
            struct packed {
                logic value;
            } rx_fifo_bad_frame;
            struct packed {
                logic value;
            } rx_fifo_good_frame;
        } mac_status;
        struct packed {
            struct packed {
                logic value;
            } mii_loopback_en;
        } mii_loopback;
    } field_storage_t;
    field_storage_t field_storage;

    // Field: mac_regs.mac_status.tx_error_underflow
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.tx_error_underflow.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.tx_error_underflow.value & ~(decoded_wr_data[0:0] & decoded_wr_biten[0:0]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.tx_error_underflow.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.tx_error_underflow.next = next_c;
        field_combo.mac_status.tx_error_underflow.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.tx_error_underflow.value <= 1'h0;
        end else if(field_combo.mac_status.tx_error_underflow.load_next) begin
            field_storage.mac_status.tx_error_underflow.value <= field_combo.mac_status.tx_error_underflow.next;
        end
    end
    assign hwif_out.mac_status.tx_error_underflow.value = field_storage.mac_status.tx_error_underflow.value;
    // Field: mac_regs.mac_status.tx_fifo_overflow
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.tx_fifo_overflow.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.tx_fifo_overflow.value & ~(decoded_wr_data[1:1] & decoded_wr_biten[1:1]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.tx_fifo_overflow.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.tx_fifo_overflow.next = next_c;
        field_combo.mac_status.tx_fifo_overflow.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.tx_fifo_overflow.value <= 1'h0;
        end else if(field_combo.mac_status.tx_fifo_overflow.load_next) begin
            field_storage.mac_status.tx_fifo_overflow.value <= field_combo.mac_status.tx_fifo_overflow.next;
        end
    end
    assign hwif_out.mac_status.tx_fifo_overflow.value = field_storage.mac_status.tx_fifo_overflow.value;
    // Field: mac_regs.mac_status.tx_fifo_bad_frame
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.tx_fifo_bad_frame.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.tx_fifo_bad_frame.value & ~(decoded_wr_data[2:2] & decoded_wr_biten[2:2]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.tx_fifo_bad_frame.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.tx_fifo_bad_frame.next = next_c;
        field_combo.mac_status.tx_fifo_bad_frame.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.tx_fifo_bad_frame.value <= 1'h0;
        end else if(field_combo.mac_status.tx_fifo_bad_frame.load_next) begin
            field_storage.mac_status.tx_fifo_bad_frame.value <= field_combo.mac_status.tx_fifo_bad_frame.next;
        end
    end
    assign hwif_out.mac_status.tx_fifo_bad_frame.value = field_storage.mac_status.tx_fifo_bad_frame.value;
    // Field: mac_regs.mac_status.tx_fifo_good_frame
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.tx_fifo_good_frame.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.tx_fifo_good_frame.value & ~(decoded_wr_data[3:3] & decoded_wr_biten[3:3]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.tx_fifo_good_frame.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.tx_fifo_good_frame.next = next_c;
        field_combo.mac_status.tx_fifo_good_frame.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.tx_fifo_good_frame.value <= 1'h0;
        end else if(field_combo.mac_status.tx_fifo_good_frame.load_next) begin
            field_storage.mac_status.tx_fifo_good_frame.value <= field_combo.mac_status.tx_fifo_good_frame.next;
        end
    end
    assign hwif_out.mac_status.tx_fifo_good_frame.value = field_storage.mac_status.tx_fifo_good_frame.value;
    // Field: mac_regs.mac_status.rx_error_bad_frame
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.rx_error_bad_frame.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.rx_error_bad_frame.value & ~(decoded_wr_data[4:4] & decoded_wr_biten[4:4]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.rx_error_bad_frame.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.rx_error_bad_frame.next = next_c;
        field_combo.mac_status.rx_error_bad_frame.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.rx_error_bad_frame.value <= 1'h0;
        end else if(field_combo.mac_status.rx_error_bad_frame.load_next) begin
            field_storage.mac_status.rx_error_bad_frame.value <= field_combo.mac_status.rx_error_bad_frame.next;
        end
    end
    assign hwif_out.mac_status.rx_error_bad_frame.value = field_storage.mac_status.rx_error_bad_frame.value;
    // Field: mac_regs.mac_status.rx_error_bad_fcs
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.rx_error_bad_fcs.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.rx_error_bad_fcs.value & ~(decoded_wr_data[5:5] & decoded_wr_biten[5:5]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.rx_error_bad_fcs.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.rx_error_bad_fcs.next = next_c;
        field_combo.mac_status.rx_error_bad_fcs.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.rx_error_bad_fcs.value <= 1'h0;
        end else if(field_combo.mac_status.rx_error_bad_fcs.load_next) begin
            field_storage.mac_status.rx_error_bad_fcs.value <= field_combo.mac_status.rx_error_bad_fcs.next;
        end
    end
    assign hwif_out.mac_status.rx_error_bad_fcs.value = field_storage.mac_status.rx_error_bad_fcs.value;
    // Field: mac_regs.mac_status.rx_fifo_overflow
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.rx_fifo_overflow.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.rx_fifo_overflow.value & ~(decoded_wr_data[6:6] & decoded_wr_biten[6:6]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.rx_fifo_overflow.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.rx_fifo_overflow.next = next_c;
        field_combo.mac_status.rx_fifo_overflow.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.rx_fifo_overflow.value <= 1'h0;
        end else if(field_combo.mac_status.rx_fifo_overflow.load_next) begin
            field_storage.mac_status.rx_fifo_overflow.value <= field_combo.mac_status.rx_fifo_overflow.next;
        end
    end
    assign hwif_out.mac_status.rx_fifo_overflow.value = field_storage.mac_status.rx_fifo_overflow.value;
    // Field: mac_regs.mac_status.rx_fifo_bad_frame
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.rx_fifo_bad_frame.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.rx_fifo_bad_frame.value & ~(decoded_wr_data[7:7] & decoded_wr_biten[7:7]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.rx_fifo_bad_frame.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.rx_fifo_bad_frame.next = next_c;
        field_combo.mac_status.rx_fifo_bad_frame.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.rx_fifo_bad_frame.value <= 1'h0;
        end else if(field_combo.mac_status.rx_fifo_bad_frame.load_next) begin
            field_storage.mac_status.rx_fifo_bad_frame.value <= field_combo.mac_status.rx_fifo_bad_frame.next;
        end
    end
    assign hwif_out.mac_status.rx_fifo_bad_frame.value = field_storage.mac_status.rx_fifo_bad_frame.value;
    // Field: mac_regs.mac_status.rx_fifo_good_frame
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mac_status.rx_fifo_good_frame.value;
        load_next_c = '0;
        if(decoded_reg_strb.mac_status && decoded_req_is_wr) begin // SW write 1 clear
            next_c = field_storage.mac_status.rx_fifo_good_frame.value & ~(decoded_wr_data[8:8] & decoded_wr_biten[8:8]);
            load_next_c = '1;
        end else if(hwif_in.mac_status.rx_fifo_good_frame.hwset) begin // HW Set
            next_c = '1;
            load_next_c = '1;
        end
        field_combo.mac_status.rx_fifo_good_frame.next = next_c;
        field_combo.mac_status.rx_fifo_good_frame.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mac_status.rx_fifo_good_frame.value <= 1'h0;
        end else if(field_combo.mac_status.rx_fifo_good_frame.load_next) begin
            field_storage.mac_status.rx_fifo_good_frame.value <= field_combo.mac_status.rx_fifo_good_frame.next;
        end
    end
    assign hwif_out.mac_status.rx_fifo_good_frame.value = field_storage.mac_status.rx_fifo_good_frame.value;
    // Field: mac_regs.mii_loopback.mii_loopback_en
    always_comb begin
        static logic [0:0] next_c;
        static logic load_next_c;
        next_c = field_storage.mii_loopback.mii_loopback_en.value;
        load_next_c = '0;
        if(decoded_reg_strb.mii_loopback && decoded_req_is_wr) begin // SW write
            next_c = (field_storage.mii_loopback.mii_loopback_en.value & ~decoded_wr_biten[0:0]) | (decoded_wr_data[0:0] & decoded_wr_biten[0:0]);
            load_next_c = '1;
        end
        field_combo.mii_loopback.mii_loopback_en.next = next_c;
        field_combo.mii_loopback.mii_loopback_en.load_next = load_next_c;
    end
    always_ff @(posedge clk) begin
        if(rst) begin
            field_storage.mii_loopback.mii_loopback_en.value <= 1'h0;
        end else if(field_combo.mii_loopback.mii_loopback_en.load_next) begin
            field_storage.mii_loopback.mii_loopback_en.value <= field_combo.mii_loopback.mii_loopback_en.next;
        end
    end
    assign hwif_out.mii_loopback.mii_loopback_en.value = field_storage.mii_loopback.mii_loopback_en.value;

    //--------------------------------------------------------------------------
    // Write response
    //--------------------------------------------------------------------------
    assign cpuif_wr_ack = decoded_req & decoded_req_is_wr;
    // Writes are always granted with no error response
    assign cpuif_wr_err = '0;

    //--------------------------------------------------------------------------
    // Readback
    //--------------------------------------------------------------------------

    logic readback_err;
    logic readback_done;
    logic [31:0] readback_data;

    // Assign readback values to a flattened array
    logic [31:0] readback_array[2];
    assign readback_array[0][0:0] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.tx_error_underflow.value : '0;
    assign readback_array[0][1:1] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.tx_fifo_overflow.value : '0;
    assign readback_array[0][2:2] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.tx_fifo_bad_frame.value : '0;
    assign readback_array[0][3:3] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.tx_fifo_good_frame.value : '0;
    assign readback_array[0][4:4] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.rx_error_bad_frame.value : '0;
    assign readback_array[0][5:5] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.rx_error_bad_fcs.value : '0;
    assign readback_array[0][6:6] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.rx_fifo_overflow.value : '0;
    assign readback_array[0][7:7] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.rx_fifo_bad_frame.value : '0;
    assign readback_array[0][8:8] = (decoded_reg_strb.mac_status && !decoded_req_is_wr) ? field_storage.mac_status.rx_fifo_good_frame.value : '0;
    assign readback_array[0][31:9] = '0;
    assign readback_array[1][0:0] = (decoded_reg_strb.mii_loopback && !decoded_req_is_wr) ? field_storage.mii_loopback.mii_loopback_en.value : '0;
    assign readback_array[1][31:1] = '0;

    // Reduce the array
    always_comb begin
        static logic [31:0] readback_data_var;
        readback_done = decoded_req & ~decoded_req_is_wr;
        readback_err = '0;
        readback_data_var = '0;
        for(int i=0; i<2; i++) readback_data_var |= readback_array[i];
        readback_data = readback_data_var;
    end

    assign cpuif_rd_ack = readback_done;
    assign cpuif_rd_data = readback_data;
    assign cpuif_rd_err = readback_err;
endmodule
