
// Checksum Modifier
module checksum_mod (
    input   [15:0]  i_checksum,
    input   [15:0]  i_old_data,
    input   [15:0]  i_new_data,
    output  [15:0]  o_checksum
);

    // The checksum modifier implements RFC 1624, Eqn 3 to update the checksum
    // after modifying a single 16 bit word in the data. It does this 
    // combinationally since it consists only of a few additions.

    logic [15:0] checksum_invert;
    logic [15:0] previous_val_invert;
    logic [31:0] new_checksum_temp;
    logic [15:0] new_checksum;

    assign checksum_invert = ~i_checksum;
    assign previous_val_invert = ~i_old_data;

    assign new_checksum_temp = (checksum_invert + previous_val_invert + i_new_data);
    assign o_checksum = ~(new_checksum_temp[31:16] + new_checksum_temp[15:0]);


endmodule