module ping_wrapper#(
    parameter DATA_WIDTH = 64,
    parameter KEEP_ENABLE = (DATA_WIDTH>8),
    parameter KEEP_WIDTH = (DATA_WIDTH/8),
    parameter USER_WIDTH = 1
)
(
    input  wire                         clk,
    input  wire                         rst,

    /*
     * IP frame input
     */
    input  wire                         s_ip_hdr_valid,
    output wire                         s_ip_hdr_ready,
    input  wire [47:0]                  s_eth_dest_mac,
    input  wire [47:0]                  s_eth_src_mac,
    input  wire [15:0]                  s_eth_type,
    input  wire [3:0]                   s_ip_version,
    input  wire [3:0]                   s_ip_ihl,
    input  wire [5:0]                   s_ip_dscp,
    input  wire [1:0]                   s_ip_ecn,
    input  wire [15:0]                  s_ip_length,
    input  wire [15:0]                  s_ip_identification,
    input  wire [2:0]                   s_ip_flags,
    input  wire [12:0]                  s_ip_fragment_offset,
    input  wire [7:0]                   s_ip_ttl,
    input  wire [7:0]                   s_ip_protocol,
    input  wire [15:0]                  s_ip_header_checksum,
    input  wire [31:0]                  s_ip_source_ip,
    input  wire [31:0]                  s_ip_dest_ip,
    input  wire [DATA_WIDTH-1:0]        s_ip_payload_axis_tdata,
    input  wire [KEEP_WIDTH-1:0]        s_ip_payload_axis_tkeep,
    input  wire                         s_ip_payload_axis_tvalid,
    output wire                         s_ip_payload_axis_tready,
    input  wire                         s_ip_payload_axis_tlast,
    input  wire [USER_WIDTH-1:0]        s_ip_payload_axis_tuser,

    /*
     * IP frame output
     */
    output wire                         m_ip_hdr_valid,
    input  wire                         m_ip_hdr_ready,
    output wire [47:0]                  m_eth_dest_mac,
    output wire [47:0]                  m_eth_src_mac,
    output wire [15:0]                  m_eth_type,
    output wire [3:0]                   m_ip_version,
    output wire [3:0]                   m_ip_ihl,
    output wire [5:0]                   m_ip_dscp,
    output wire [1:0]                   m_ip_ecn,
    output wire [15:0]                  m_ip_length,
    output wire [15:0]                  m_ip_identification,
    output wire [2:0]                   m_ip_flags,
    output wire [12:0]                  m_ip_fragment_offset,
    output wire [7:0]                   m_ip_ttl,
    output wire [7:0]                   m_ip_protocol,
    output wire [15:0]                  m_ip_header_checksum,
    output wire [31:0]                  m_ip_source_ip,
    output wire [31:0]                  m_ip_dest_ip,
    output wire [DATA_WIDTH-1:0]        m_ip_payload_axis_tdata,
    output wire [KEEP_WIDTH-1:0]        m_ip_payload_axis_tkeep,
    output wire                         m_ip_payload_axis_tvalid,
    input  wire                         m_ip_payload_axis_tready,
    output wire                         m_ip_payload_axis_tlast,
    output wire                         m_ip_payload_axis_tuser
);


    logic                       ping_rx_icmp_hdr_valid;
    logic                       ping_rx_icmp_hdr_ready;
    logic [47:0]                ping_rx_eth_dest_mac;
    logic [47:0]                ping_rx_eth_src_mac;
    logic [15:0]                ping_rx_eth_type;
    logic [3:0]                 ping_rx_ip_version;
    logic [3:0]                 ping_rx_ip_ihl;
    logic [5:0]                 ping_rx_ip_dscp;
    logic [1:0]                 ping_rx_ip_ecn;
    logic [15:0]                ping_rx_ip_length;
    logic [15:0]                ping_rx_ip_identification;
    logic [2:0]                 ping_rx_ip_flags;
    logic [12:0]                ping_rx_ip_fragment_offset;
    logic [7:0]                 ping_rx_ip_ttl;
    logic [7:0]                 ping_rx_ip_protocol;
    logic [15:0]                ping_rx_ip_header_checksum;
    logic [31:0]                ping_rx_ip_source_ip;
    logic [31:0]                ping_rx_ip_dest_ip;
    logic [7:0]                 ping_rx_icmp_type;
    logic [7:0]                 ping_rx_icmp_code;
    logic [15:0]                ping_rx_icmp_checksum;
    logic [31:0]                ping_rx_icmp_rest_of_header;
    logic [DATA_WIDTH-1:0]      ping_rx_icmp_payload_axis_tdata;
    logic [KEEP_WIDTH-1:0]      ping_rx_icmp_payload_axis_tkeep;
    logic                       ping_rx_icmp_payload_axis_tvalid;
    logic                       ping_rx_icmp_payload_axis_tready;
    logic                       ping_rx_icmp_payload_axis_tlast;
    logic                       ping_rx_icmp_payload_axis_tuser;

    logic                       ping_rx_busy;
    logic                       ping_rx_error_header_early_termination;
    logic                       ping_rx_error_payload_early_termination;

    logic                       ping_tx_icmp_hdr_valid;
    logic                       ping_tx_icmp_hdr_ready;
    logic [47:0]                ping_tx_eth_dest_mac;
    logic [47:0]                ping_tx_eth_src_mac;
    logic [15:0]                ping_tx_eth_type;
    logic [3:0]                 ping_tx_ip_version;
    logic [3:0]                 ping_tx_ip_ihl;
    logic [5:0]                 ping_tx_ip_dscp;
    logic [1:0]                 ping_tx_ip_ecn;
    logic [15:0]                ping_tx_ip_length;
    logic [15:0]                ping_tx_ip_identification;
    logic [2:0]                 ping_tx_ip_flags;
    logic [12:0]                ping_tx_ip_fragment_offset;
    logic [7:0]                 ping_tx_ip_ttl;
    logic [7:0]                 ping_tx_ip_protocol;
    logic [15:0]                ping_tx_ip_header_checksum;
    logic [31:0]                ping_tx_ip_source_ip;
    logic [31:0]                ping_tx_ip_dest_ip;
    logic [7:0]                 ping_tx_icmp_type;
    logic [7:0]                 ping_tx_icmp_code;
    logic [15:0]                ping_tx_icmp_checksum;
    logic [31:0]                ping_tx_icmp_rest_of_header;
    logic [DATA_WIDTH-1:0]      ping_tx_icmp_payload_axis_tdata;
    logic [KEEP_WIDTH-1:0]      ping_tx_icmp_payload_axis_tkeep;
    logic                       ping_tx_icmp_payload_axis_tvalid;
    logic                       ping_tx_icmp_payload_axis_tready;
    logic                       ping_tx_icmp_payload_axis_tlast;
    logic                       ping_tx_icmp_payload_axis_tuser;

    logic ping_tx_busy;
    logic ping_tx_error_payload_early_termination;


generate
    if (DATA_WIDTH == 64) begin
        icmp_ip_rx_64 u_ping_wrapper_icmp_ip_rx_64 (
            .clk                                (clk),
            .rst                                (rst),

            .s_ip_hdr_valid                     (s_ip_hdr_valid),
            .s_ip_hdr_ready                     (s_ip_hdr_ready),
            .s_eth_dest_mac                     (s_eth_dest_mac),
            .s_eth_src_mac                      (s_eth_src_mac),
            .s_eth_type                         (s_eth_type),
            .s_ip_version                       (s_ip_version),
            .s_ip_ihl                           (s_ip_ihl),
            .s_ip_dscp                          (s_ip_dscp),
            .s_ip_ecn                           (s_ip_ecn),
            .s_ip_length                        (s_ip_length),
            .s_ip_identification                (s_ip_identification),
            .s_ip_flags                         (s_ip_flags),
            .s_ip_fragment_offset               (s_ip_fragment_offset),
            .s_ip_ttl                           (s_ip_ttl),
            .s_ip_protocol                      (s_ip_protocol),
            .s_ip_header_checksum               (s_ip_header_checksum),
            .s_ip_source_ip                     (s_ip_source_ip),
            .s_ip_dest_ip                       (s_ip_dest_ip),
            .s_ip_payload_axis_tdata            (s_ip_payload_axis_tdata),
            .s_ip_payload_axis_tkeep            (s_ip_payload_axis_tkeep),
            .s_ip_payload_axis_tvalid           (s_ip_payload_axis_tvalid),
            .s_ip_payload_axis_tready           (s_ip_payload_axis_tready),
            .s_ip_payload_axis_tlast            (s_ip_payload_axis_tlast),
            .s_ip_payload_axis_tuser            (s_ip_payload_axis_tuser),

            .m_icmp_hdr_valid                   (ping_rx_icmp_hdr_valid),
            .m_icmp_hdr_ready                   (ping_rx_icmp_hdr_ready),
            .m_eth_dest_mac                     (ping_rx_eth_dest_mac),
            .m_eth_src_mac                      (ping_rx_eth_src_mac),
            .m_eth_type                         (ping_rx_eth_type),
            .m_ip_version                       (ping_rx_ip_version),
            .m_ip_ihl                           (ping_rx_ip_ihl),
            .m_ip_dscp                          (ping_rx_ip_dscp),
            .m_ip_ecn                           (ping_rx_ip_ecn),
            .m_ip_length                        (ping_rx_ip_length),
            .m_ip_identification                (ping_rx_ip_identification),
            .m_ip_flags                         (ping_rx_ip_flags),
            .m_ip_fragment_offset               (ping_rx_ip_fragment_offset),
            .m_ip_ttl                           (ping_rx_ip_ttl),
            .m_ip_protocol                      (ping_rx_ip_protocol),
            .m_ip_header_checksum               (ping_rx_ip_header_checksum),
            .m_ip_source_ip                     (ping_rx_ip_source_ip),
            .m_ip_dest_ip                       (ping_rx_ip_dest_ip),
            .m_icmp_type                        (ping_rx_icmp_type),
            .m_icmp_code                        (ping_rx_icmp_code),
            .m_icmp_checksum                    (ping_rx_icmp_checksum),
            .m_icmp_rest_of_header              (ping_rx_icmp_rest_of_header),
            .m_icmp_payload_axis_tdata          (ping_rx_icmp_payload_axis_tdata),
            .m_icmp_payload_axis_tkeep          (ping_rx_icmp_payload_axis_tkeep),
            .m_icmp_payload_axis_tvalid         (ping_rx_icmp_payload_axis_tvalid),
            .m_icmp_payload_axis_tready         (ping_rx_icmp_payload_axis_tready),
            .m_icmp_payload_axis_tlast          (ping_rx_icmp_payload_axis_tlast),
            .m_icmp_payload_axis_tuser          (ping_rx_icmp_payload_axis_tuser),

            .busy                               (ping_rx_busy),
            .error_header_early_termination     (ping_rx_error_header_early_termination),
            .error_payload_early_termination    (ping_rx_error_payload_early_termination)
        );
    end else if (DATA_WIDTH == 8) begin
        icmp_ip_rx u_ping_wrapper_icmp_ip_rx (
            .clk                                (clk),
            .rst                                (rst),

            .s_ip_hdr_valid                     (s_ip_hdr_valid),
            .s_ip_hdr_ready                     (s_ip_hdr_ready),
            .s_eth_dest_mac                     (s_eth_dest_mac),
            .s_eth_src_mac                      (s_eth_src_mac),
            .s_eth_type                         (s_eth_type),
            .s_ip_version                       (s_ip_version),
            .s_ip_ihl                           (s_ip_ihl),
            .s_ip_dscp                          (s_ip_dscp),
            .s_ip_ecn                           (s_ip_ecn),
            .s_ip_length                        (s_ip_length),
            .s_ip_identification                (s_ip_identification),
            .s_ip_flags                         (s_ip_flags),
            .s_ip_fragment_offset               (s_ip_fragment_offset),
            .s_ip_ttl                           (s_ip_ttl),
            .s_ip_protocol                      (s_ip_protocol),
            .s_ip_header_checksum               (s_ip_header_checksum),
            .s_ip_source_ip                     (s_ip_source_ip),
            .s_ip_dest_ip                       (s_ip_dest_ip),
            .s_ip_payload_axis_tdata            (s_ip_payload_axis_tdata),
            .s_ip_payload_axis_tvalid           (s_ip_payload_axis_tvalid),
            .s_ip_payload_axis_tready           (s_ip_payload_axis_tready),
            .s_ip_payload_axis_tlast            (s_ip_payload_axis_tlast),
            .s_ip_payload_axis_tuser            (s_ip_payload_axis_tuser),

            .m_icmp_hdr_valid                   (ping_rx_icmp_hdr_valid),
            .m_icmp_hdr_ready                   (ping_rx_icmp_hdr_ready),
            .m_eth_dest_mac                     (ping_rx_eth_dest_mac),
            .m_eth_src_mac                      (ping_rx_eth_src_mac),
            .m_eth_type                         (ping_rx_eth_type),
            .m_ip_version                       (ping_rx_ip_version),
            .m_ip_ihl                           (ping_rx_ip_ihl),
            .m_ip_dscp                          (ping_rx_ip_dscp),
            .m_ip_ecn                           (ping_rx_ip_ecn),
            .m_ip_length                        (ping_rx_ip_length),
            .m_ip_identification                (ping_rx_ip_identification),
            .m_ip_flags                         (ping_rx_ip_flags),
            .m_ip_fragment_offset               (ping_rx_ip_fragment_offset),
            .m_ip_ttl                           (ping_rx_ip_ttl),
            .m_ip_protocol                      (ping_rx_ip_protocol),
            .m_ip_header_checksum               (ping_rx_ip_header_checksum),
            .m_ip_source_ip                     (ping_rx_ip_source_ip),
            .m_ip_dest_ip                       (ping_rx_ip_dest_ip),
            .m_icmp_type                        (ping_rx_icmp_type),
            .m_icmp_code                        (ping_rx_icmp_code),
            .m_icmp_checksum                    (ping_rx_icmp_checksum),
            .m_icmp_rest_of_header              (ping_rx_icmp_rest_of_header),
            .m_icmp_payload_axis_tdata          (ping_rx_icmp_payload_axis_tdata),
            .m_icmp_payload_axis_tvalid         (ping_rx_icmp_payload_axis_tvalid),
            .m_icmp_payload_axis_tready         (ping_rx_icmp_payload_axis_tready),
            .m_icmp_payload_axis_tlast          (ping_rx_icmp_payload_axis_tlast),
            .m_icmp_payload_axis_tuser          (ping_rx_icmp_payload_axis_tuser),

            .busy                               (ping_rx_busy),
            .error_header_early_termination     (ping_rx_error_header_early_termination),
            .error_payload_early_termination    (ping_rx_error_payload_early_termination)
        );

        assign ping_rx_icmp_payload_axis_tkeep = ping_rx_icmp_payload_axis_tvalid;

    end else begin
        $error("Only 8 and 64 bit data widths are supported");
    end
endgenerate

    ping_client #(.DATA_WIDTH(DATA_WIDTH)) u_ping_client (
        .clk                                (clk),
        .rst                                (rst),

        .s_icmp_hdr_valid                   (ping_rx_icmp_hdr_valid),
        .s_icmp_hdr_ready                   (ping_rx_icmp_hdr_ready),
        .s_eth_dest_mac                     (ping_rx_eth_dest_mac),
        .s_eth_src_mac                      (ping_rx_eth_src_mac),
        .s_eth_type                         (ping_rx_eth_type),
        .s_ip_version                       (ping_rx_ip_version),
        .s_ip_ihl                           (ping_rx_ip_ihl),
        .s_ip_dscp                          (ping_rx_ip_dscp),
        .s_ip_ecn                           (ping_rx_ip_ecn),
        .s_ip_length                        (ping_rx_ip_length),
        .s_ip_identification                (ping_rx_ip_identification),
        .s_ip_flags                         (ping_rx_ip_flags),
        .s_ip_fragment_offset               (ping_rx_ip_fragment_offset),
        .s_ip_ttl                           (ping_rx_ip_ttl),
        .s_ip_protocol                      (ping_rx_ip_protocol),
        .s_ip_header_checksum               (ping_rx_ip_header_checksum),
        .s_ip_source_ip                     (ping_rx_ip_source_ip),
        .s_ip_dest_ip                       (ping_rx_ip_dest_ip),
        .s_icmp_type                        (ping_rx_icmp_type),
        .s_icmp_code                        (ping_rx_icmp_code),
        .s_icmp_checksum                    (ping_rx_icmp_checksum),
        .s_icmp_rest_of_header              (ping_rx_icmp_rest_of_header),
        .s_icmp_payload_axis_tdata          (ping_rx_icmp_payload_axis_tdata),
        .s_icmp_payload_axis_tkeep          (ping_rx_icmp_payload_axis_tkeep),
        .s_icmp_payload_axis_tvalid         (ping_rx_icmp_payload_axis_tvalid),
        .s_icmp_payload_axis_tready         (ping_rx_icmp_payload_axis_tready),
        .s_icmp_payload_axis_tlast          (ping_rx_icmp_payload_axis_tlast),
        .s_icmp_payload_axis_tuser          (ping_rx_icmp_payload_axis_tuser),

        .m_icmp_hdr_valid                   (ping_tx_icmp_hdr_valid),
        .m_icmp_hdr_ready                   (ping_tx_icmp_hdr_ready),
        .m_eth_dest_mac                     (ping_tx_eth_dest_mac),
        .m_eth_src_mac                      (ping_tx_eth_src_mac),
        .m_eth_type                         (ping_tx_eth_type),
        .m_ip_version                       (ping_tx_ip_version),
        .m_ip_ihl                           (ping_tx_ip_ihl),
        .m_ip_dscp                          (ping_tx_ip_dscp),
        .m_ip_ecn                           (ping_tx_ip_ecn),
        .m_ip_length                        (ping_tx_ip_length),
        .m_ip_identification                (ping_tx_ip_identification),
        .m_ip_flags                         (ping_tx_ip_flags),
        .m_ip_fragment_offset               (ping_tx_ip_fragment_offset),
        .m_ip_ttl                           (ping_tx_ip_ttl),
        .m_ip_protocol                      (ping_tx_ip_protocol),
        .m_ip_header_checksum               (ping_tx_ip_header_checksum),
        .m_ip_source_ip                     (ping_tx_ip_source_ip),
        .m_ip_dest_ip                       (ping_tx_ip_dest_ip),
        .m_icmp_type                        (ping_tx_icmp_type),
        .m_icmp_code                        (ping_tx_icmp_code),
        .m_icmp_checksum                    (ping_tx_icmp_checksum),
        .m_icmp_rest_of_header              (ping_tx_icmp_rest_of_header),
        .m_icmp_payload_axis_tdata          (ping_tx_icmp_payload_axis_tdata),
        .m_icmp_payload_axis_tkeep          (ping_tx_icmp_payload_axis_tkeep),
        .m_icmp_payload_axis_tvalid         (ping_tx_icmp_payload_axis_tvalid),
        .m_icmp_payload_axis_tready         (ping_tx_icmp_payload_axis_tready),
        .m_icmp_payload_axis_tlast          (ping_tx_icmp_payload_axis_tlast),
        .m_icmp_payload_axis_tuser          (ping_tx_icmp_payload_axis_tuser)
    );

generate
    if (DATA_WIDTH == 64) begin
        icmp_ip_tx_64 u_ping_wrapper_icmp_ip_tx_64 (
            .clk                                (clk),
            .rst                                (rst),

            .s_icmp_hdr_valid                   (ping_tx_icmp_hdr_valid),
            .s_icmp_hdr_ready                   (ping_tx_icmp_hdr_ready),
            .s_eth_dest_mac                     (ping_tx_eth_dest_mac),
            .s_eth_src_mac                      (ping_tx_eth_src_mac),
            .s_eth_type                         (ping_tx_eth_type),
            .s_ip_version                       (ping_tx_ip_version),
            .s_ip_ihl                           (ping_tx_ip_ihl),
            .s_ip_dscp                          (ping_tx_ip_dscp),
            .s_ip_ecn                           (ping_tx_ip_ecn),
            .s_ip_length                        (ping_tx_ip_length),
            .s_ip_identification                (ping_tx_ip_identification),
            .s_ip_flags                         (ping_tx_ip_flags),
            .s_ip_fragment_offset               (ping_tx_ip_fragment_offset),
            .s_ip_ttl                           (ping_tx_ip_ttl),
            .s_ip_protocol                      (ping_tx_ip_protocol),
            .s_ip_header_checksum               (ping_tx_ip_header_checksum),
            .s_ip_source_ip                     (ping_tx_ip_source_ip),
            .s_ip_dest_ip                       (ping_tx_ip_dest_ip),
            .s_icmp_type                        (ping_tx_icmp_type),
            .s_icmp_code                        (ping_tx_icmp_code),
            .s_icmp_checksum                    (ping_tx_icmp_checksum),
            .s_icmp_rest_of_header              (ping_tx_icmp_rest_of_header),
            .s_icmp_payload_axis_tdata          (ping_tx_icmp_payload_axis_tdata),
            .s_icmp_payload_axis_tkeep          (ping_tx_icmp_payload_axis_tkeep),
            .s_icmp_payload_axis_tvalid         (ping_tx_icmp_payload_axis_tvalid),
            .s_icmp_payload_axis_tready         (ping_tx_icmp_payload_axis_tready),
            .s_icmp_payload_axis_tlast          (ping_tx_icmp_payload_axis_tlast),
            .s_icmp_payload_axis_tuser          (ping_tx_icmp_payload_axis_tuser),

            .m_ip_hdr_valid                     (m_ip_hdr_valid),
            .m_ip_hdr_ready                     (m_ip_hdr_ready),
            .m_eth_dest_mac                     (m_eth_dest_mac),
            .m_eth_src_mac                      (m_eth_src_mac),
            .m_eth_type                         (m_eth_type),
            .m_ip_version                       (m_ip_version),
            .m_ip_ihl                           (m_ip_ihl),
            .m_ip_dscp                          (m_ip_dscp),
            .m_ip_ecn                           (m_ip_ecn),
            .m_ip_length                        (m_ip_length),
            .m_ip_identification                (m_ip_identification),
            .m_ip_flags                         (m_ip_flags),
            .m_ip_fragment_offset               (m_ip_fragment_offset),
            .m_ip_ttl                           (m_ip_ttl),
            .m_ip_protocol                      (m_ip_protocol),
            .m_ip_header_checksum               (m_ip_header_checksum),
            .m_ip_source_ip                     (m_ip_source_ip),
            .m_ip_dest_ip                       (m_ip_dest_ip),
            .m_ip_payload_axis_tdata            (m_ip_payload_axis_tdata),
            .m_ip_payload_axis_tkeep            (m_ip_payload_axis_tkeep),
            .m_ip_payload_axis_tvalid           (m_ip_payload_axis_tvalid),
            .m_ip_payload_axis_tready           (m_ip_payload_axis_tready),
            .m_ip_payload_axis_tlast            (m_ip_payload_axis_tlast),
            .m_ip_payload_axis_tuser            (m_ip_payload_axis_tuser),

            .busy                               (ping_tx_busy),
            .error_payload_early_termination    (ping_tx_error_payload_early_termination)
        );
    end else if (DATA_WIDTH == 8) begin
        icmp_ip_tx u_ping_wrapper_icmp_ip_tx (
            .clk                                (clk),
            .rst                                (rst),

            .s_icmp_hdr_valid                   (ping_tx_icmp_hdr_valid),
            .s_icmp_hdr_ready                   (ping_tx_icmp_hdr_ready),
            .s_eth_dest_mac                     (ping_tx_eth_dest_mac),
            .s_eth_src_mac                      (ping_tx_eth_src_mac),
            .s_eth_type                         (ping_tx_eth_type),
            .s_ip_version                       (ping_tx_ip_version),
            .s_ip_ihl                           (ping_tx_ip_ihl),
            .s_ip_dscp                          (ping_tx_ip_dscp),
            .s_ip_ecn                           (ping_tx_ip_ecn),
            .s_ip_length                        (ping_tx_ip_length),
            .s_ip_identification                (ping_tx_ip_identification),
            .s_ip_flags                         (ping_tx_ip_flags),
            .s_ip_fragment_offset               (ping_tx_ip_fragment_offset),
            .s_ip_ttl                           (ping_tx_ip_ttl),
            .s_ip_protocol                      (ping_tx_ip_protocol),
            .s_ip_header_checksum               (ping_tx_ip_header_checksum),
            .s_ip_source_ip                     (ping_tx_ip_source_ip),
            .s_ip_dest_ip                       (ping_tx_ip_dest_ip),
            .s_icmp_type                        (ping_tx_icmp_type),
            .s_icmp_code                        (ping_tx_icmp_code),
            .s_icmp_checksum                    (ping_tx_icmp_checksum),
            .s_icmp_rest_of_header              (ping_tx_icmp_rest_of_header),
            .s_icmp_payload_axis_tdata          (ping_tx_icmp_payload_axis_tdata),
            .s_icmp_payload_axis_tvalid         (ping_tx_icmp_payload_axis_tvalid),
            .s_icmp_payload_axis_tready         (ping_tx_icmp_payload_axis_tready),
            .s_icmp_payload_axis_tlast          (ping_tx_icmp_payload_axis_tlast),
            .s_icmp_payload_axis_tuser          (ping_tx_icmp_payload_axis_tuser),

            .m_ip_hdr_valid                     (m_ip_hdr_valid),
            .m_ip_hdr_ready                     (m_ip_hdr_ready),
            .m_eth_dest_mac                     (m_eth_dest_mac),
            .m_eth_src_mac                      (m_eth_src_mac),
            .m_eth_type                         (m_eth_type),
            .m_ip_version                       (m_ip_version),
            .m_ip_ihl                           (m_ip_ihl),
            .m_ip_dscp                          (m_ip_dscp),
            .m_ip_ecn                           (m_ip_ecn),
            .m_ip_length                        (m_ip_length),
            .m_ip_identification                (m_ip_identification),
            .m_ip_flags                         (m_ip_flags),
            .m_ip_fragment_offset               (m_ip_fragment_offset),
            .m_ip_ttl                           (m_ip_ttl),
            .m_ip_protocol                      (m_ip_protocol),
            .m_ip_header_checksum               (m_ip_header_checksum),
            .m_ip_source_ip                     (m_ip_source_ip),
            .m_ip_dest_ip                       (m_ip_dest_ip),
            .m_ip_payload_axis_tdata            (m_ip_payload_axis_tdata),
            .m_ip_payload_axis_tvalid           (m_ip_payload_axis_tvalid),
            .m_ip_payload_axis_tready           (m_ip_payload_axis_tready),
            .m_ip_payload_axis_tlast            (m_ip_payload_axis_tlast),
            .m_ip_payload_axis_tuser            (m_ip_payload_axis_tuser),

            .busy                               (ping_tx_busy),
            .error_payload_early_termination    (ping_tx_error_payload_early_termination)
        );

        assign m_ip_payload_axis_tkeep = m_ip_payload_axis_tvalid;
    end else begin
        $error("Only 8 and 64 bit data widths are supported");
    end
endgenerate

endmodule